@@ -324,6 +324,7 @@ fiber_pool_vacancy_push(struct fiber_pool_vacancy * vacancy, struct fiber_pool_v
 #ifdef FIBER_POOL_ALLOCATION_FREE
     if (head) {
         head->previous = vacancy;
+        vacancy->previous = NULL;
     }
 #endif
 
@@ -919,9 +920,7 @@ cont_free(void *ptr)
     else {
         rb_fiber_t *fiber = (rb_fiber_t*)cont;
         coroutine_destroy(&fiber->context);
-        if (!fiber_is_root_p(fiber)) {
-            fiber_stack_release(fiber);
-        }
+        fiber_stack_release(fiber);
     }
 
     RUBY_FREE_UNLESS_NULL(cont->saved_vm_stack.ptr);
@@ -1096,6 +1095,15 @@ cont_save_thread(rb_context_t *cont, rb_thread_t *th)
     sec->machine.stack_end = NULL;
 }
 
+static void
+cont_init_mjit_cont(rb_context_t *cont)
+{
+    VM_ASSERT(cont->mjit_cont == NULL);
+    if (mjit_enabled) {
+        cont->mjit_cont = mjit_cont_new(&(cont->saved_ec));
+    }
+}
+
 static void
 cont_init(rb_context_t *cont, rb_thread_t *th)
 {
@@ -1105,9 +1113,7 @@ cont_init(rb_context_t *cont, rb_thread_t *th)
     cont->saved_ec.local_storage = NULL;
     cont->saved_ec.local_storage_recursive_hash = Qnil;
     cont->saved_ec.local_storage_recursive_hash_for_trace = Qnil;
-    if (mjit_enabled) {
-        cont->mjit_cont = mjit_cont_new(&cont->saved_ec);
-    }
+    cont_init_mjit_cont(cont);
 }
 
 static rb_context_t *
@@ -1124,6 +1130,14 @@ cont_new(VALUE klass)
     return cont;
 }
 
+void
+rb_fiber_init_mjit_cont(struct rb_fiber_struct *fiber)
+{
+    // Currently this function is meant for root_fiber. Others go through cont_new.
+    // XXX: Is this mjit_cont `mjit_cont_free`d?
+    cont_init_mjit_cont(&fiber->cont);
+}
+
 #if 0
 void
 show_vm_stack(const rb_execution_context_t *ec)
