@@ -128,7 +128,9 @@ rb_syntax_error_append(VALUE exc, VALUE file, int line, int column,
     return exc;
 }
 
-static unsigned int warning_disabled_categories;
+static unsigned int warning_disabled_categories = (
+    1U << RB_WARN_CATEGORY_DEPRECATED |
+    0);
 
 static unsigned int
 rb_warning_category_mask(VALUE category)
