@@ -79,6 +79,47 @@ error_print(rb_execution_context_t *ec)
     rb_ec_error_print(ec, ec->errinfo);
 }
 
+static void
+write_warnq(VALUE out, VALUE str, const char *ptr, long len)
+{
+    if (NIL_P(out)) {
+        const char *beg = ptr;
+        const long olen = len;
+        for (; len > 0; --len, ++ptr) {
+            unsigned char c = *ptr;
+            switch (c) {
+              case '\n': case '\t': continue;
+            }
+            if (rb_iscntrl(c)) {
+                char buf[5];
+                const char *cc = 0;
+                if (ptr > beg) rb_write_error2(beg, ptr - beg);
+                beg = ptr + 1;
+                cc = ruby_escaped_char(c);
+                if (cc) {
+                    rb_write_error2(cc, strlen(cc));
+                }
+                else {
+                    rb_write_error2(buf, snprintf(buf, sizeof(buf), "\\x%02X", c));
+                }
+            }
+            else if (c == '\\') {
+                rb_write_error2(beg, ptr - beg + 1);
+                beg = ptr;
+            }
+        }
+        if (ptr > beg) {
+            if (beg == RSTRING_PTR(str) && olen == RSTRING_LEN(str))
+                rb_write_error_str(str);
+            else
+                rb_write_error2(beg, ptr - beg);
+        }
+    }
+    else {
+        rb_str_cat(out, ptr, len);
+    }
+}
+
 #define CSI_BEGIN "\033["
 #define CSI_SGR "m"
 
@@ -103,8 +144,6 @@ print_errinfo(const VALUE eclass, const VALUE errat, const VALUE emesg, const VA
 	    write_warn(str, ": ");
 	}
 
-	if (highlight) write_warn(str, bold);
-
 	if (!NIL_P(emesg)) {
 	    einfo = RSTRING_PTR(emesg);
             elen = RSTRING_LEN(emesg);
@@ -128,17 +167,18 @@ print_errinfo(const VALUE eclass, const VALUE errat, const VALUE emesg, const VA
 	    write_warn(str, "\n");
 	}
 	else {
+            /* emesg is a String instance */
 	    const char *tail = 0;
 
-	    if (emesg == Qundef && highlight) write_warn(str, bold);
+            if (highlight) write_warn(str, bold);
 	    if (RSTRING_PTR(epath)[0] == '#')
 		epath = 0;
 	    if ((tail = memchr(einfo, '\n', elen)) != 0) {
-		write_warn2(str, einfo, tail - einfo);
+                write_warnq(str, emesg, einfo, tail - einfo);
 		tail++;		/* skip newline */
 	    }
 	    else {
-		write_warn_str(str, emesg);
+                write_warnq(str, emesg, einfo, elen);
 	    }
 	    if (epath) {
 		write_warn(str, " (");
@@ -154,7 +194,7 @@ print_errinfo(const VALUE eclass, const VALUE errat, const VALUE emesg, const VA
 	    }
 	    if (tail && einfo+elen > tail) {
 		if (!highlight) {
-		    write_warn2(str, tail, einfo+elen-tail);
+                    write_warnq(str, emesg, tail, einfo+elen-tail);
 		    if (einfo[elen-1] != '\n') write_warn2(str, "\n", 1);
 		}
 		else {
@@ -164,7 +204,7 @@ print_errinfo(const VALUE eclass, const VALUE errat, const VALUE emesg, const VA
 			tail = memchr(einfo, '\n', elen);
 			if (!tail || tail > einfo) {
 			    write_warn(str, bold);
-			    write_warn2(str, einfo, tail ? tail-einfo : elen);
+                            write_warnq(str, emesg, einfo, tail ? tail-einfo : elen);
 			    write_warn(str, reset);
 			    if (!tail) {
 				write_warn2(str, "\n", 1);
@@ -174,7 +214,7 @@ print_errinfo(const VALUE eclass, const VALUE errat, const VALUE emesg, const VA
 			elen -= tail - einfo;
 			einfo = tail;
 			do ++tail; while (tail < einfo+elen && *tail == '\n');
-			write_warn2(str, einfo, tail-einfo);
+                        write_warnq(str, emesg, einfo, tail-einfo);
 			elen -= tail - einfo;
 			einfo = tail;
 		    }
@@ -308,6 +348,7 @@ rb_ec_error_print(rb_execution_context_t * volatile ec, volatile VALUE errinfo)
     volatile uint8_t raised_flag = ec->raised_flag;
     volatile VALUE errat = Qundef;
     volatile VALUE emesg = Qundef;
+    volatile bool written = false;
 
     if (NIL_P(errinfo))
 	return;
@@ -322,7 +363,10 @@ rb_ec_error_print(rb_execution_context_t * volatile ec, volatile VALUE errinfo)
 	emesg = rb_get_message(errinfo);
     }
 
-    rb_error_write(errinfo, emesg, errat, Qnil, Qnil, Qnil);
+    if (!written) {
+        written = true;
+        rb_error_write(errinfo, emesg, errat, Qnil, Qnil, Qnil);
+    }
 
     EC_POP_TAG();
     ec->errinfo = errinfo;
@@ -389,10 +433,9 @@ sysexit_status(VALUE err)
     rb_bug("Unknown longjmp status %d", status)
 
 static int
-error_handle(int ex)
+error_handle(rb_execution_context_t *ec, int ex)
 {
     int status = EXIT_FAILURE;
-    rb_execution_context_t *ec = GET_EC();
 
     if (rb_ec_set_raised(ec))
 	return EXIT_FAILURE;
