@@ -11,7 +11,7 @@ normalize_ospath(VALUE str)
 #endif
 
 void
-Init_normalize(VALUE klass)
+Init_string_normalize(VALUE klass)
 {
     rb_define_method(klass, "normalize_ospath", normalize_ospath, 0);
 }
