@@ -77,8 +77,8 @@ RUBY_EXTERN void Init_digest_base(void);
  *
  * Different digest algorithms (or hash functions) are available:
  *
- * HMAC::
- *   See FIPS PUB 198 The Keyed-Hash Message Authentication Code (HMAC).
+ * MD5::
+ *  See RFC 1321 The MD5 Message-Digest Algorithm
  * RIPEMD-160::
  *   As Digest::RMD160.
  *   See http://homes.esat.kuleuven.be/~bosselae/ripemd160.html.
@@ -486,7 +486,7 @@ rb_digest_class_s_digest(int argc, VALUE *argv, VALUE klass)
 static VALUE
 rb_digest_class_s_hexdigest(int argc, VALUE *argv, VALUE klass)
 {
-    return hexencode_str_new(rb_funcall2(klass, id_digest, argc, argv));
+    return hexencode_str_new(rb_funcallv(klass, id_digest, argc, argv));
 }
 
 /* :nodoc: */
@@ -501,6 +501,40 @@ rb_digest_class_init(VALUE self)
  *
  * This abstract class provides a common interface to message digest
  * implementation classes written in C.
+ *
+ * ==Write a Digest subclass in C
+ * Digest::Base provides a common interface to message digest
+ * classes written in C. These classes must provide a struct
+ * of type rb_digest_metadata_t:
+ *  typedef int (*rb_digest_hash_init_func_t)(void *);
+ *  typedef void (*rb_digest_hash_update_func_t)(void *, unsigned char *, size_t);
+ *  typedef int (*rb_digest_hash_finish_func_t)(void *, unsigned char *);
+ *
+ *  typedef struct {
+ *    int api_version;
+ *    size_t digest_len;
+ *    size_t block_len;
+ *    size_t ctx_size;
+ *    rb_digest_hash_init_func_t init_func;
+ *    rb_digest_hash_update_func_t update_func;
+ *    rb_digest_hash_finish_func_t finish_func;
+ *  } rb_digest_metadata_t;
+ *
+ * This structure must be set as an instance variable named +metadata+
+ * (without the +@+ in front of the name). By example:
+ *   static const rb_digest_metadata_t sha1 = {
+ *      RUBY_DIGEST_API_VERSION,
+ *      SHA1_DIGEST_LENGTH,
+ *      SHA1_BLOCK_LENGTH,
+ *      sizeof(SHA1_CTX),
+ *      (rb_digest_hash_init_func_t)SHA1_Init,
+ *      (rb_digest_hash_update_func_t)SHA1_Update,
+ *      (rb_digest_hash_finish_func_t)SHA1_Finish,
+ *  };
+ *
+ *
+ *  rb_ivar_set(cDigest_SHA1, rb_intern("metadata"),
+ *		Data_Wrap_Struct(0, 0, 0, (void *)&sha1));
  */
 
 static rb_digest_metadata_t *
@@ -520,10 +554,22 @@ get_digest_base_metadata(VALUE klass)
     if (NIL_P(p))
         rb_raise(rb_eRuntimeError, "Digest::Base cannot be directly inherited in Ruby");
 
+    if (!RB_TYPE_P(obj, T_DATA) || RTYPEDDATA_P(obj)) {
+      wrong:
+        if (p == klass)
+            rb_raise(rb_eTypeError, "%"PRIsVALUE"::metadata is not initialized properly",
+                     klass);
+        else
+            rb_raise(rb_eTypeError, "%"PRIsVALUE"(%"PRIsVALUE")::metadata is not initialized properly",
+                     klass, p);
+    }
+
 #undef RUBY_UNTYPED_DATA_WARNING
 #define RUBY_UNTYPED_DATA_WARNING 0
     Data_Get_Struct(obj, rb_digest_metadata_t, algo);
 
+    if (!algo) goto wrong;
+
     switch (algo->api_version) {
       case 3:
         break;
@@ -539,6 +585,12 @@ get_digest_base_metadata(VALUE klass)
     return algo;
 }
 
+static rb_digest_metadata_t *
+get_digest_obj_metadata(VALUE obj)
+{
+    return get_digest_base_metadata(rb_obj_class(obj));
+}
+
 static const rb_data_type_t digest_type = {
     "digest",
     {0, RUBY_TYPED_DEFAULT_FREE, 0,},
@@ -585,8 +637,8 @@ rb_digest_base_copy(VALUE copy, VALUE obj)
 
     rb_check_frozen(copy);
 
-    algo = get_digest_base_metadata(rb_obj_class(copy));
-    if (algo != get_digest_base_metadata(rb_obj_class(obj)))
+    algo = get_digest_obj_metadata(copy);
+    if (algo != get_digest_obj_metadata(obj))
 	rb_raise(rb_eTypeError, "different algorithms");
 
     TypedData_Get_Struct(obj, void, &digest_type, pctx1);
@@ -596,14 +648,18 @@ rb_digest_base_copy(VALUE copy, VALUE obj)
     return copy;
 }
 
-/* :nodoc: */
+/*
+ * call-seq: digest_base.reset -> digest_base
+ *
+ * Reset the digest to its initial state and return +self+.
+ */
 static VALUE
 rb_digest_base_reset(VALUE self)
 {
     rb_digest_metadata_t *algo;
     void *pctx;
 
-    algo = get_digest_base_metadata(rb_obj_class(self));
+    algo = get_digest_obj_metadata(self);
 
     TypedData_Get_Struct(self, void, &digest_type, pctx);
 
@@ -612,14 +668,20 @@ rb_digest_base_reset(VALUE self)
     return self;
 }
 
-/* :nodoc: */
+/*
+ * call-seq:
+ *   digest_base.update(string) -> digest_base
+ *   digest_base << string -> digest_base
+ *
+ * Update the digest using given _string_ and return +self+.
+ */
 static VALUE
 rb_digest_base_update(VALUE self, VALUE str)
 {
     rb_digest_metadata_t *algo;
     void *pctx;
 
-    algo = get_digest_base_metadata(rb_obj_class(self));
+    algo = get_digest_obj_metadata(self);
 
     TypedData_Get_Struct(self, void, &digest_type, pctx);
 
@@ -638,7 +700,7 @@ rb_digest_base_finish(VALUE self)
     void *pctx;
     VALUE str;
 
-    algo = get_digest_base_metadata(rb_obj_class(self));
+    algo = get_digest_obj_metadata(self);
 
     TypedData_Get_Struct(self, void, &digest_type, pctx);
 
@@ -651,24 +713,32 @@ rb_digest_base_finish(VALUE self)
     return str;
 }
 
-/* :nodoc: */
+/*
+ * call-seq: digest_base.digest_length -> Integer
+ *
+ * Return the length of the hash value in bytes.
+ */
 static VALUE
 rb_digest_base_digest_length(VALUE self)
 {
     rb_digest_metadata_t *algo;
 
-    algo = get_digest_base_metadata(rb_obj_class(self));
+    algo = get_digest_obj_metadata(self);
 
     return INT2NUM(algo->digest_len);
 }
 
-/* :nodoc: */
+/*
+ * call-seq: digest_base.block_length -> Integer
+ *
+ * Return the block length of the digest in bytes.
+ */
 static VALUE
 rb_digest_base_block_length(VALUE self)
 {
     rb_digest_metadata_t *algo;
 
-    algo = get_digest_base_metadata(rb_obj_class(self));
+    algo = get_digest_obj_metadata(self);
 
     return INT2NUM(algo->block_len);
 }
@@ -676,13 +746,20 @@ rb_digest_base_block_length(VALUE self)
 void
 Init_digest(void)
 {
+#undef rb_intern
     id_reset           = rb_intern("reset");
     id_update          = rb_intern("update");
     id_finish          = rb_intern("finish");
     id_digest          = rb_intern("digest");
     id_hexdigest       = rb_intern("hexdigest");
     id_digest_length   = rb_intern("digest_length");
+    id_metadata        = rb_id_metadata();
+    InitVM(digest);
+}
 
+void
+InitVM_digest(void)
+{
     /*
      * module Digest
      */
@@ -729,8 +806,6 @@ Init_digest(void)
     rb_define_singleton_method(rb_cDigest_Class, "digest", rb_digest_class_s_digest, -1);
     rb_define_singleton_method(rb_cDigest_Class, "hexdigest", rb_digest_class_s_hexdigest, -1);
 
-    id_metadata = rb_intern("metadata");
-
     /* class Digest::Base < Digest::Class */
     rb_cDigest_Base = rb_define_class_under(rb_mDigest, "Base", rb_cDigest_Class);
 
