@@ -8,7 +8,7 @@
 
 #define MD5_BLOCK_LENGTH	MD5_CBLOCK
 
-static DEFINE_FINISH_FUNC_FROM_FINAL(MD5);
+static DEFINE_FINISH_FUNC_FROM_FINAL(MD5)
 #undef MD5_Finish
 #define MD5_Finish rb_digest_MD5_finish
 
