@@ -12,27 +12,30 @@
 
 **********************************************************************/
 
+#include "ruby/io.h"
 #include "internal.h"
 #include "ruby/debug.h"
-#include "ruby/io.h"
 #include "gc.h"
 #include "node.h"
 #include "vm_core.h"
 #include "objspace.h"
 
 static VALUE sym_output, sym_stdout, sym_string, sym_file;
+static VALUE sym_full;
 
 struct dump_config {
     VALUE type;
     FILE *stream;
     VALUE string;
-    int roots;
     const char *root_category;
     VALUE cur_obj;
     VALUE cur_obj_klass;
     size_t cur_obj_references;
+    unsigned int roots: 1;
+    unsigned int full_heap: 1;
 };
 
+PRINTF_ARGS(static void dump_append(struct dump_config *, const char *, ...), 2, 3);
 static void
 dump_append(struct dump_config *dc, const char *format, ...)
 {
@@ -102,7 +105,7 @@ static inline const char *
 obj_type(VALUE obj)
 {
     switch (BUILTIN_TYPE(obj)) {
-#define CASE_TYPE(type) case T_##type: return #type; break
+#define CASE_TYPE(type) case T_##type: return #type
 	CASE_TYPE(NONE);
 	CASE_TYPE(NIL);
 	CASE_TYPE(OBJECT);
@@ -169,9 +172,9 @@ reachable_object_i(VALUE ref, void *data)
 	return;
 
     if (dc->cur_obj_references == 0)
-	dump_append(dc, ", \"references\":[\"%p\"", (void *)ref);
+        dump_append(dc, ", \"references\":[\"%#"PRIxVALUE"\"", ref);
     else
-	dump_append(dc, ", \"%p\"", (void *)ref);
+        dump_append(dc, ", \"%#"PRIxVALUE"\"", ref);
 
     dc->cur_obj_references++;
 }
@@ -181,7 +184,7 @@ dump_append_string_content(struct dump_config *dc, VALUE obj)
 {
     dump_append(dc, ", \"bytesize\":%ld", RSTRING_LEN(obj));
     if (!STR_EMBED_P(obj) && !STR_SHARED_P(obj) && (long)rb_str_capacity(obj) != RSTRING_LEN(obj))
-	dump_append(dc, ", \"capacity\":%ld", rb_str_capacity(obj));
+	dump_append(dc, ", \"capacity\":%"PRIuSIZE, rb_str_capacity(obj));
 
     if (is_ascii_string(obj)) {
 	dump_append(dc, ", \"value\":");
@@ -189,6 +192,28 @@ dump_append_string_content(struct dump_config *dc, VALUE obj)
     }
 }
 
+static const char *
+imemo_name(int imemo)
+{
+    switch(imemo) {
+#define TYPE_STR(t) case(imemo_##t): return #t
+	TYPE_STR(env);
+	TYPE_STR(cref);
+	TYPE_STR(svar);
+	TYPE_STR(throw_data);
+	TYPE_STR(ifunc);
+	TYPE_STR(memo);
+	TYPE_STR(ment);
+	TYPE_STR(iseq);
+	TYPE_STR(tmpbuf);
+	TYPE_STR(ast);
+	TYPE_STR(parser_strterm);
+      default:
+	return "unknown";
+#undef TYPE_STR
+    }
+}
+
 static void
 dump_object(VALUE obj, struct dump_config *dc)
 {
@@ -210,16 +235,20 @@ dump_object(VALUE obj, struct dump_config *dc)
     if (dc->cur_obj == dc->string)
 	return;
 
-    dump_append(dc, "{\"address\":\"%p\", \"type\":\"%s\"", (void *)obj, obj_type(obj));
+    dump_append(dc, "{\"address\":\"%#"PRIxVALUE"\", \"type\":\"%s\"", obj, obj_type(obj));
 
     if (dc->cur_obj_klass)
-	dump_append(dc, ", \"class\":\"%p\"", (void *)dc->cur_obj_klass);
+        dump_append(dc, ", \"class\":\"%#"PRIxVALUE"\"", dc->cur_obj_klass);
     if (rb_obj_frozen_p(obj))
 	dump_append(dc, ", \"frozen\":true");
 
     switch (BUILTIN_TYPE(obj)) {
-      case T_NODE:
-	dump_append(dc, ", \"node_type\":\"%s\"", ruby_node_name(nd_type(obj)));
+      case T_NONE:
+	dump_append(dc, "}\n");
+	return;
+
+      case T_IMEMO:
+	dump_append(dc, ", \"imemo_type\":\"%s\"", imemo_name(imemo_type(obj)));
 	break;
 
       case T_SYMBOL:
@@ -243,9 +272,9 @@ dump_object(VALUE obj, struct dump_config *dc)
 	break;
 
       case T_HASH:
-	dump_append(dc, ", \"size\":%ld", RHASH_SIZE(obj));
-	if (FL_TEST(obj, HASH_PROC_DEFAULT))
-	    dump_append(dc, ", \"default\":\"%p\"", (void *)RHASH_IFNONE(obj));
+	dump_append(dc, ", \"size\":%"PRIuSIZE, (size_t)RHASH_SIZE(obj));
+        if (FL_TEST(obj, RHASH_PROC_DEFAULT))
+            dump_append(dc, ", \"default\":\"%#"PRIxVALUE"\"", RHASH_IFNONE(obj));
 	break;
 
       case T_ARRAY:
@@ -272,7 +301,7 @@ dump_object(VALUE obj, struct dump_config *dc)
 	break;
 
       case T_OBJECT:
-	dump_append(dc, ", \"ivars\":%ld", ROBJECT_NUMIV(obj));
+	dump_append(dc, ", \"ivars\":%u", ROBJECT_NUMIV(obj));
 	break;
 
       case T_FILE:
@@ -317,10 +346,11 @@ dump_object(VALUE obj, struct dump_config *dc)
 static int
 heap_i(void *vstart, void *vend, size_t stride, void *data)
 {
+    struct dump_config *dc = (struct dump_config *)data;
     VALUE v = (VALUE)vstart;
     for (; v != (VALUE)vend; v += stride) {
-	if (RBASIC(v)->flags)
-	    dump_object(v, data);
+	if (dc->full_heap || RBASIC(v)->flags)
+	    dump_object(v, dc);
     }
     return 0;
 }
@@ -333,12 +363,12 @@ root_obj_i(const char *category, VALUE obj, void *data)
     if (dc->root_category != NULL && category != dc->root_category)
 	dump_append(dc, "]}\n");
     if (dc->root_category == NULL || category != dc->root_category)
-	dump_append(dc, "{\"type\":\"ROOT\", \"root\":\"%s\", \"references\":[\"%p\"", category, (void *)obj);
+        dump_append(dc, "{\"type\":\"ROOT\", \"root\":\"%s\", \"references\":[\"%#"PRIxVALUE"\"", category, obj);
     else
-	dump_append(dc, ", \"%p\"", (void *)obj);
+        dump_append(dc, ", \"%#"PRIxVALUE"\"", obj);
 
     dc->root_category = category;
-    dc->roots++;
+    dc->roots = 1;
 }
 
 static VALUE
@@ -346,9 +376,15 @@ dump_output(struct dump_config *dc, VALUE opts, VALUE output, const char *filena
 {
     VALUE tmp;
 
-    if (RTEST(opts))
+    dc->full_heap = 0;
+
+    if (RTEST(opts)) {
 	output = rb_hash_aref(opts, sym_output);
 
+	if (Qtrue == rb_hash_lookup2(opts, sym_full, Qfalse))
+	    dc->full_heap = 1;
+    }
+
     if (output == sym_stdout) {
 	dc->stream = stdout;
 	dc->string = Qnil;
@@ -381,7 +417,7 @@ static VALUE
 dump_result(struct dump_config *dc, VALUE output)
 {
     if (output == sym_string) {
-	return dc->string;
+	return rb_str_resurrect(dc->string);
     }
     else if (output == sym_file) {
 	rb_io_flush(dc->string);
@@ -462,6 +498,7 @@ objspace_dump_all(int argc, VALUE *argv, VALUE os)
 void
 Init_objspace_dump(VALUE rb_mObjSpace)
 {
+#undef rb_intern
 #if 0
     rb_mObjSpace = rb_define_module("ObjectSpace"); /* let rdoc know */
 #endif
@@ -473,6 +510,7 @@ Init_objspace_dump(VALUE rb_mObjSpace)
     sym_stdout = ID2SYM(rb_intern("stdout"));
     sym_string = ID2SYM(rb_intern("string"));
     sym_file   = ID2SYM(rb_intern("file"));
+    sym_full   = ID2SYM(rb_intern("full"));
 
     /* force create static IDs */
     rb_obj_gc_flags(rb_mObjSpace, 0, 0);
