@@ -11,7 +11,7 @@
 
 */
 
-#include "ruby/ruby.h"
+#include <ruby.h>
 
 #ifndef FALSE
 #define FALSE 0
@@ -24,7 +24,7 @@
                         Important Constants
 ----------------------------------------------------------------------- */
 
-#define RACC_VERSION "1.4.5"
+#define RACC_VERSION "1.4.15"
 
 #define DEFAULT_TOKEN -1
 #define ERROR_TOKEN    1
@@ -72,6 +72,10 @@ static ID id_d_e_pop;
 #  define LONG2NUM(i) INT2NUM(i)
 #endif
 
+#ifndef HAVE_RB_ARY_SUBSEQ
+#  define rb_ary_subseq(ary, beg, len) rb_ary_new4(len, RARRAY_PTR(ary) + beg)
+#endif
+
 static ID value_to_id _((VALUE v));
 static inline long num_to_long _((VALUE n));
 
@@ -208,7 +212,7 @@ static void extract_user_token _((struct cparse_params *v,
                                   VALUE block_args, VALUE *tok, VALUE *val));
 static void shift _((struct cparse_params* v, long act, VALUE tok, VALUE val));
 static int reduce _((struct cparse_params* v, long act));
-static VALUE reduce0 _((VALUE block_args, VALUE data, VALUE self));
+static rb_block_call_func reduce0;
 
 #ifdef DEBUG
 # define D_puts(msg)        if (v->sys_debug) puts(msg)
@@ -704,7 +708,7 @@ reduce(struct cparse_params *v, long act)
 }
 
 static VALUE
-reduce0(VALUE val, VALUE data, VALUE self)
+reduce0(RB_BLOCK_CALL_FUNC_ARGLIST(_, data))
 {
     struct cparse_params *v = rb_check_typeddata(data, &cparse_params_type);
     VALUE reduce_to, reduce_len, method_id;
