@@ -10,8 +10,10 @@ static struct {
     ID id_args_add;
     ID id_args_add_block;
     ID id_args_add_star;
+    ID id_args_forward;
     ID id_args_new;
     ID id_array;
+    ID id_aryptn;
     ID id_assign;
     ID id_assign_error;
     ID id_assoc_new;
@@ -21,8 +23,6 @@ static struct {
     ID id_begin;
     ID id_binary;
     ID id_block_var;
-    ID id_block_var_add_block;
-    ID id_block_var_add_star;
     ID id_blockarg;
     ID id_bodystmt;
     ID id_brace_block;
@@ -51,15 +51,20 @@ static struct {
     ID id_field;
     ID id_for;
     ID id_hash;
+    ID id_heredoc_dedent;
+    ID id_hshptn;
     ID id_if;
     ID id_if_mod;
     ID id_ifop;
+    ID id_in;
+    ID id_kwrest_param;
     ID id_lambda;
     ID id_magic_comment;
     ID id_massign;
     ID id_method_add_arg;
     ID id_method_add_block;
     ID id_mlhs_add;
+    ID id_mlhs_add_post;
     ID id_mlhs_add_star;
     ID id_mlhs_new;
     ID id_mlhs_paren;
@@ -69,6 +74,7 @@ static struct {
     ID id_mrhs_new;
     ID id_mrhs_new_from_args;
     ID id_next;
+    ID id_nokw_param;
     ID id_opassign;
     ID id_operator_ambiguous;
     ID id_param_error;
@@ -143,8 +149,10 @@ static struct {
 #define ripper_id_args_add ripper_parser_ids.id_args_add
 #define ripper_id_args_add_block ripper_parser_ids.id_args_add_block
 #define ripper_id_args_add_star ripper_parser_ids.id_args_add_star
+#define ripper_id_args_forward ripper_parser_ids.id_args_forward
 #define ripper_id_args_new ripper_parser_ids.id_args_new
 #define ripper_id_array ripper_parser_ids.id_array
+#define ripper_id_aryptn ripper_parser_ids.id_aryptn
 #define ripper_id_assign ripper_parser_ids.id_assign
 #define ripper_id_assign_error ripper_parser_ids.id_assign_error
 #define ripper_id_assoc_new ripper_parser_ids.id_assoc_new
@@ -154,8 +162,6 @@ static struct {
 #define ripper_id_begin ripper_parser_ids.id_begin
 #define ripper_id_binary ripper_parser_ids.id_binary
 #define ripper_id_block_var ripper_parser_ids.id_block_var
-#define ripper_id_block_var_add_block ripper_parser_ids.id_block_var_add_block
-#define ripper_id_block_var_add_star ripper_parser_ids.id_block_var_add_star
 #define ripper_id_blockarg ripper_parser_ids.id_blockarg
 #define ripper_id_bodystmt ripper_parser_ids.id_bodystmt
 #define ripper_id_brace_block ripper_parser_ids.id_brace_block
@@ -184,15 +190,20 @@ static struct {
 #define ripper_id_field ripper_parser_ids.id_field
 #define ripper_id_for ripper_parser_ids.id_for
 #define ripper_id_hash ripper_parser_ids.id_hash
+#define ripper_id_heredoc_dedent ripper_parser_ids.id_heredoc_dedent
+#define ripper_id_hshptn ripper_parser_ids.id_hshptn
 #define ripper_id_if ripper_parser_ids.id_if
 #define ripper_id_if_mod ripper_parser_ids.id_if_mod
 #define ripper_id_ifop ripper_parser_ids.id_ifop
+#define ripper_id_in ripper_parser_ids.id_in
+#define ripper_id_kwrest_param ripper_parser_ids.id_kwrest_param
 #define ripper_id_lambda ripper_parser_ids.id_lambda
 #define ripper_id_magic_comment ripper_parser_ids.id_magic_comment
 #define ripper_id_massign ripper_parser_ids.id_massign
 #define ripper_id_method_add_arg ripper_parser_ids.id_method_add_arg
 #define ripper_id_method_add_block ripper_parser_ids.id_method_add_block
 #define ripper_id_mlhs_add ripper_parser_ids.id_mlhs_add
+#define ripper_id_mlhs_add_post ripper_parser_ids.id_mlhs_add_post
 #define ripper_id_mlhs_add_star ripper_parser_ids.id_mlhs_add_star
 #define ripper_id_mlhs_new ripper_parser_ids.id_mlhs_new
 #define ripper_id_mlhs_paren ripper_parser_ids.id_mlhs_paren
@@ -202,6 +213,7 @@ static struct {
 #define ripper_id_mrhs_new ripper_parser_ids.id_mrhs_new
 #define ripper_id_mrhs_new_from_args ripper_parser_ids.id_mrhs_new_from_args
 #define ripper_id_next ripper_parser_ids.id_next
+#define ripper_id_nokw_param ripper_parser_ids.id_nokw_param
 #define ripper_id_opassign ripper_parser_ids.id_opassign
 #define ripper_id_operator_ambiguous ripper_parser_ids.id_operator_ambiguous
 #define ripper_id_param_error ripper_parser_ids.id_param_error
@@ -279,8 +291,10 @@ ripper_init_eventids1(void)
     set_id1(args_add);
     set_id1(args_add_block);
     set_id1(args_add_star);
+    set_id1(args_forward);
     set_id1(args_new);
     set_id1(array);
+    set_id1(aryptn);
     set_id1(assign);
     set_id1(assign_error);
     set_id1(assoc_new);
@@ -290,8 +304,6 @@ ripper_init_eventids1(void)
     set_id1(begin);
     set_id1(binary);
     set_id1(block_var);
-    set_id1(block_var_add_block);
-    set_id1(block_var_add_star);
     set_id1(blockarg);
     set_id1(bodystmt);
     set_id1(brace_block);
@@ -320,15 +332,20 @@ ripper_init_eventids1(void)
     set_id1(field);
     set_id1(for);
     set_id1(hash);
+    set_id1(heredoc_dedent);
+    set_id1(hshptn);
     set_id1(if);
     set_id1(if_mod);
     set_id1(ifop);
+    set_id1(in);
+    set_id1(kwrest_param);
     set_id1(lambda);
     set_id1(magic_comment);
     set_id1(massign);
     set_id1(method_add_arg);
     set_id1(method_add_block);
     set_id1(mlhs_add);
+    set_id1(mlhs_add_post);
     set_id1(mlhs_add_star);
     set_id1(mlhs_new);
     set_id1(mlhs_paren);
@@ -338,6 +355,7 @@ ripper_init_eventids1(void)
     set_id1(mrhs_new);
     set_id1(mrhs_new_from_args);
     set_id1(next);
+    set_id1(nokw_param);
     set_id1(opassign);
     set_id1(operator_ambiguous);
     set_id1(param_error);
@@ -417,8 +435,10 @@ ripper_init_eventids1_table(VALUE self)
     rb_hash_aset(h, intern_sym("args_add"), INT2FIX(2));
     rb_hash_aset(h, intern_sym("args_add_block"), INT2FIX(2));
     rb_hash_aset(h, intern_sym("args_add_star"), INT2FIX(2));
+    rb_hash_aset(h, intern_sym("args_forward"), INT2FIX(0));
     rb_hash_aset(h, intern_sym("args_new"), INT2FIX(0));
     rb_hash_aset(h, intern_sym("array"), INT2FIX(1));
+    rb_hash_aset(h, intern_sym("aryptn"), INT2FIX(4));
     rb_hash_aset(h, intern_sym("assign"), INT2FIX(2));
     rb_hash_aset(h, intern_sym("assign_error"), INT2FIX(1));
     rb_hash_aset(h, intern_sym("assoc_new"), INT2FIX(2));
@@ -428,8 +448,6 @@ ripper_init_eventids1_table(VALUE self)
     rb_hash_aset(h, intern_sym("begin"), INT2FIX(1));
     rb_hash_aset(h, intern_sym("binary"), INT2FIX(3));
     rb_hash_aset(h, intern_sym("block_var"), INT2FIX(2));
-    rb_hash_aset(h, intern_sym("block_var_add_block"), INT2FIX(2));
-    rb_hash_aset(h, intern_sym("block_var_add_star"), INT2FIX(2));
     rb_hash_aset(h, intern_sym("blockarg"), INT2FIX(1));
     rb_hash_aset(h, intern_sym("bodystmt"), INT2FIX(4));
     rb_hash_aset(h, intern_sym("brace_block"), INT2FIX(2));
@@ -453,20 +471,25 @@ ripper_init_eventids1_table(VALUE self)
     rb_hash_aset(h, intern_sym("else"), INT2FIX(1));
     rb_hash_aset(h, intern_sym("elsif"), INT2FIX(3));
     rb_hash_aset(h, intern_sym("ensure"), INT2FIX(1));
-    rb_hash_aset(h, intern_sym("excessed_comma"), INT2FIX(1));
+    rb_hash_aset(h, intern_sym("excessed_comma"), INT2FIX(0));
     rb_hash_aset(h, intern_sym("fcall"), INT2FIX(1));
     rb_hash_aset(h, intern_sym("field"), INT2FIX(3));
     rb_hash_aset(h, intern_sym("for"), INT2FIX(3));
     rb_hash_aset(h, intern_sym("hash"), INT2FIX(1));
+    rb_hash_aset(h, intern_sym("heredoc_dedent"), INT2FIX(2));
+    rb_hash_aset(h, intern_sym("hshptn"), INT2FIX(3));
     rb_hash_aset(h, intern_sym("if"), INT2FIX(3));
     rb_hash_aset(h, intern_sym("if_mod"), INT2FIX(2));
     rb_hash_aset(h, intern_sym("ifop"), INT2FIX(3));
+    rb_hash_aset(h, intern_sym("in"), INT2FIX(3));
+    rb_hash_aset(h, intern_sym("kwrest_param"), INT2FIX(1));
     rb_hash_aset(h, intern_sym("lambda"), INT2FIX(2));
     rb_hash_aset(h, intern_sym("magic_comment"), INT2FIX(2));
     rb_hash_aset(h, intern_sym("massign"), INT2FIX(2));
     rb_hash_aset(h, intern_sym("method_add_arg"), INT2FIX(2));
     rb_hash_aset(h, intern_sym("method_add_block"), INT2FIX(2));
     rb_hash_aset(h, intern_sym("mlhs_add"), INT2FIX(2));
+    rb_hash_aset(h, intern_sym("mlhs_add_post"), INT2FIX(2));
     rb_hash_aset(h, intern_sym("mlhs_add_star"), INT2FIX(2));
     rb_hash_aset(h, intern_sym("mlhs_new"), INT2FIX(0));
     rb_hash_aset(h, intern_sym("mlhs_paren"), INT2FIX(1));
@@ -476,6 +499,7 @@ ripper_init_eventids1_table(VALUE self)
     rb_hash_aset(h, intern_sym("mrhs_new"), INT2FIX(0));
     rb_hash_aset(h, intern_sym("mrhs_new_from_args"), INT2FIX(1));
     rb_hash_aset(h, intern_sym("next"), INT2FIX(1));
+    rb_hash_aset(h, intern_sym("nokw_param"), INT2FIX(1));
     rb_hash_aset(h, intern_sym("opassign"), INT2FIX(3));
     rb_hash_aset(h, intern_sym("operator_ambiguous"), INT2FIX(2));
     rb_hash_aset(h, intern_sym("param_error"), INT2FIX(1));
