@@ -1,4 +1,4 @@
-# frozen_string_literal: false
+# frozen_string_literal: true
 # $Id$
 
 require 'optparse'
@@ -68,7 +68,7 @@ def usage(msg)
 end
 
 def generate_eventids1(ids)
-  buf = ""
+  buf = "".dup
   buf << %Q[static struct {\n]
   ids.each do |id, arity|
     buf << %Q[    ID id_#{id};\n]
@@ -101,7 +101,7 @@ def generate_eventids1(ids)
 end
 
 def generate_eventids2_table(ids)
-  buf = ""
+  buf = "".dup
   buf << %Q[static void\n]
   buf << %Q[ripper_init_eventids2_table(VALUE self)\n]
   buf << %Q[{\n]
@@ -135,6 +135,8 @@ def check_arity(h)
   abort if invalid
 end
 
+require_relative "dsl"
+
 def read_ids1_with_locations(path)
   h = {}
   File.open(path) {|f|
@@ -144,6 +146,13 @@ def read_ids1_with_locations(path)
       line.scan(/\bdispatch(\d)\((\w+)/) do |arity, event|
         (h[event] ||= []).push [f.lineno, arity.to_i]
       end
+      if line =~ %r</\*% *ripper(?:\[(.*?)\])?: *(.*?) *%\*/>
+        gen = DSL.new($2, ($1 || "").split(","))
+        gen.generate
+        gen.events.each do |event, arity|
+          (h[event] ||= []).push [f.lineno, arity.to_i]
+        end
+      end
     end
   }
   h
