@@ -272,6 +272,8 @@ sockopt_int(VALUE self)
  * Creates a new Socket::Option object which contains boolean as data.
  * Actually 0 or 1 as int is used.
  *
+ *   require 'socket'
+ *
  *   p Socket::Option.bool(:INET, :SOCKET, :KEEPALIVE, true)
  *   #=> #<Socket::Option: INET SOCKET KEEPALIVE 1>
  *
@@ -404,7 +406,7 @@ sockopt_s_ipv4_multicast_loop(VALUE klass, VALUE value)
  * call-seq:
  *   sockopt.ipv4_multicast_loop => integer
  *
- * Returns the ipv4_multicast_loop data in _sockopt_ as a integer.
+ * Returns the ipv4_multicast_loop data in _sockopt_ as an integer.
  *
  *   sockopt = Socket::Option.ipv4_multicast_loop(10)
  *   p sockopt.ipv4_multicast_loop => 10
@@ -422,7 +424,7 @@ sockopt_ipv4_multicast_loop(VALUE self)
     }
 #endif
     rb_raise(rb_eTypeError, "ipv4_multicast_loop socket option expected");
-    UNREACHABLE;
+    UNREACHABLE_RETURN(Qnil);
 }
 
 #define inspect_ipv4_multicast_loop(a,b,c,d) \
@@ -455,7 +457,7 @@ sockopt_s_ipv4_multicast_ttl(VALUE klass, VALUE value)
  * call-seq:
  *   sockopt.ipv4_multicast_ttl => integer
  *
- * Returns the ipv4_multicast_ttl data in _sockopt_ as a integer.
+ * Returns the ipv4_multicast_ttl data in _sockopt_ as an integer.
  *
  *   sockopt = Socket::Option.ipv4_multicast_ttl(10)
  *   p sockopt.ipv4_multicast_ttl => 10
@@ -473,7 +475,7 @@ sockopt_ipv4_multicast_ttl(VALUE self)
     }
 #endif
     rb_raise(rb_eTypeError, "ipv4_multicast_ttl socket option expected");
-    UNREACHABLE;
+    UNREACHABLE_RETURN(Qnil);
 }
 
 #define inspect_ipv4_multicast_ttl(a,b,c,d) \
@@ -645,7 +647,7 @@ inspect_timeval_as_interval(int level, int optname, VALUE data, VALUE ret)
  */
 
 #if !defined HAVE_INET_NTOP && ! defined _WIN32
-static const char *
+const char *
 inet_ntop(int af, const void *addr, char *numaddr, size_t numaddr_len)
 {
 #ifdef HAVE_INET_NTOA
@@ -660,10 +662,6 @@ inet_ntop(int af, const void *addr, char *numaddr, size_t numaddr_len)
 #endif
     return numaddr;
 }
-#elif defined __MINGW32__
-# define inet_ntop(f,a,n,l)      rb_w32_inet_ntop(f,a,n,l)
-#elif defined _MSC_VER && RUBY_MSVCRT_VERSION < 90
-const char *WSAAPI inet_ntop(int, const void *, char *, size_t);
 #endif
 
 /* Although the buffer size needed depends on the prefixes, "%u" may generate "4294967295".  */
@@ -1477,4 +1475,3 @@ rsock_init_sockopt(void)
 
     rb_define_method(rb_cSockOpt, "to_s", sockopt_data, 0); /* compatibility for ruby before 1.9.2 */
 }
-
