@@ -19,6 +19,8 @@
  * _address_family_ should be an integer, a string or a symbol:
  * Socket::AF_INET, "AF_INET", :INET, etc.
  *
+ *   require 'socket'
+ *
  *   UDPSocket.new                   #=> #<UDPSocket:fd 3>
  *   UDPSocket.new(Socket::AF_INET6) #=> #<UDPSocket:fd 4>
  *
@@ -48,8 +50,9 @@ struct udp_arg
 };
 
 static VALUE
-udp_connect_internal(struct udp_arg *arg)
+udp_connect_internal(VALUE v)
 {
+    struct udp_arg *arg = (void *)v;
     rb_io_t *fptr;
     int fd;
     struct addrinfo *res;
@@ -95,8 +98,9 @@ udp_connect(VALUE sock, VALUE host, VALUE port)
 }
 
 static VALUE
-udp_bind_internal(struct udp_arg *arg)
+udp_bind_internal(VALUE v)
 {
+    struct udp_arg *arg = (void *)v;
     rb_io_t *fptr;
     int fd;
     struct addrinfo *res;
@@ -145,8 +149,9 @@ struct udp_send_arg {
 };
 
 static VALUE
-udp_send_internal(struct udp_send_arg *arg)
+udp_send_internal(VALUE v)
 {
+    struct udp_send_arg *arg = (void *)v;
     rb_io_t *fptr;
     int n;
     struct addrinfo *res;
