@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 # = delegate -- Support for the Delegation Pattern
 #
 # Documentation by James Edward Gray II and Gavin Sinclair
@@ -17,15 +18,10 @@
 # yourself needing this control, have a look at Forwardable which is also in
 # the standard library.  It may suit your needs better.)
 #
-# SimpleDelegator's implementation serves as a nice example if the use of
+# SimpleDelegator's implementation serves as a nice example of the use of
 # Delegator:
 #
 #   class SimpleDelegator < Delegator
-#     def initialize(obj)
-#       super                  # pass obj to Delegator constructor, required
-#       @delegate_sd_obj = obj # store obj for future use
-#     end
-#
 #     def __getobj__
 #       @delegate_sd_obj # return object we are delegating to, required
 #     end
@@ -44,11 +40,11 @@ class Delegator < BasicObject
   kernel = ::Kernel.dup
   kernel.class_eval do
     alias __raise__ raise
-    [:to_s,:inspect,:=~,:!~,:===,:<=>,:eql?,:hash].each do |m|
+    [:to_s, :inspect, :=~, :!~, :===, :<=>, :hash].each do |m|
       undef_method m
     end
     private_instance_methods.each do |m|
-      if /\Ablock_given\?\z|iterator\?\z|\A__.*__\z/ =~ m
+      if /\Ablock_given\?\z|\Aiterator\?\z|\A__.*__\z/ =~ m
         next
       end
       undef_method m
@@ -62,6 +58,12 @@ class Delegator < BasicObject
   end
   # :startdoc:
 
+  ##
+  # :method: raise
+  # Use #__raise__ if your Delegator does not have a object to delegate the
+  # #raise method call.
+  #
+
   #
   # Pass in the _obj_ to delegate method calls to.  All methods supported by
   # _obj_ will be delegated to.
@@ -73,19 +75,16 @@ class Delegator < BasicObject
   #
   # Handles the magic of delegation through \_\_getobj\_\_.
   #
-  def method_missing(m, *args, &block)
+  ruby2_keywords def method_missing(m, *args, &block)
     r = true
     target = self.__getobj__ {r = false}
-    begin
-      if r && target.respond_to?(m)
-        target.__send__(m, *args, &block)
-      elsif ::Kernel.respond_to?(m, true)
-        ::Kernel.instance_method(m).bind(self).(*args, &block)
-      else
-        super(m, *args, &block)
-      end
-    ensure
-      $@.delete_if {|t| %r"\A#{Regexp.quote(__FILE__)}:(?:#{[__LINE__-7, __LINE__-5, __LINE__-3].join('|')}):"o =~ t} if $@
+
+    if r && target_respond_to?(target, m, false)
+      target.__send__(m, *args, &block)
+    elsif ::Kernel.method_defined?(m) || ::Kernel.private_method_defined?(m)
+      ::Kernel.instance_method(m).bind_call(self, *args, &block)
+    else
+      super(m, *args, &block)
     end
   end
 
@@ -96,14 +95,31 @@ class Delegator < BasicObject
   def respond_to_missing?(m, include_private)
     r = true
     target = self.__getobj__ {r = false}
-    r &&= target.respond_to?(m, include_private)
-    if r && include_private && !target.respond_to?(m, false)
-      warn "#{caller(3)[0]}: delegator does not forward private method \##{m}"
+    r &&= target_respond_to?(target, m, include_private)
+    if r && include_private && !target_respond_to?(target, m, false)
+      warn "delegator does not forward private method \##{m}", uplevel: 3
       return false
     end
     r
   end
 
+  KERNEL_RESPOND_TO = ::Kernel.instance_method(:respond_to?)
+  private_constant :KERNEL_RESPOND_TO
+
+  # Handle BasicObject instances
+  private def target_respond_to?(target, m, include_private)
+    case target
+    when Object
+      target.respond_to?(m, include_private)
+    else
+      if KERNEL_RESPOND_TO.bind_call(target, :respond_to?)
+        target.respond_to?(m, include_private)
+      else
+        KERNEL_RESPOND_TO.bind_call(target, m, include_private)
+      end
+    end
+  end
+
   #
   # Returns the methods available to this delegate object as the union
   # of this object's and \_\_getobj\_\_ methods.
@@ -146,6 +162,14 @@ class Delegator < BasicObject
     __getobj__ != obj
   end
 
+  #
+  # Returns true if two objects are considered of equal value.
+  #
+  def eql?(obj)
+    return true if obj.equal?(self)
+    obj.eql?(__getobj__)
+  end
+
   #
   # Delegates ! to the \_\_getobj\_\_
   #
@@ -176,7 +200,7 @@ class Delegator < BasicObject
     ivars = instance_variables.reject {|var| /\A@delegate_/ =~ var}
     [
       :__v2__,
-      ivars, ivars.map{|var| instance_variable_get(var)},
+      ivars, ivars.map {|var| instance_variable_get(var)},
       __getobj__
     ]
   end
@@ -187,7 +211,7 @@ class Delegator < BasicObject
   def marshal_load(data)
     version, vars, values, obj = data
     if version == :__v2__
-      vars.each_with_index{|var, i| instance_variable_set(var, values[i])}
+      vars.each_with_index {|var, i| instance_variable_set(var, values[i])}
       __setobj__(obj)
     else
       __setobj__(data)
@@ -202,40 +226,17 @@ class Delegator < BasicObject
   end
   private :initialize_clone, :initialize_dup
 
-  ##
-  # :method: trust
-  # Trust both the object returned by \_\_getobj\_\_ and self.
-  #
-
-  ##
-  # :method: untrust
-  # Untrust both the object returned by \_\_getobj\_\_ and self.
-  #
-
-  ##
-  # :method: taint
-  # Taint both the object returned by \_\_getobj\_\_ and self.
-  #
-
-  ##
-  # :method: untaint
-  # Untaint both the object returned by \_\_getobj\_\_ and self.
-  #
-
   ##
   # :method: freeze
   # Freeze both the object returned by \_\_getobj\_\_ and self.
   #
-
-  [:trust, :untrust, :taint, :untaint, :freeze].each do |method|
-    define_method method do
-      __getobj__.send(method)
-      super()
-    end
+  def freeze
+    __getobj__.freeze
+    super()
   end
 
   @delegator_api = self.public_instance_methods
-  def self.public_api   # :nodoc:
+  def self.public_api # :nodoc:
     @delegator_api
   end
 end
@@ -248,7 +249,7 @@ end
 #
 #   class User
 #     def born_on
-#       Date.new(1989, 09, 10)
+#       Date.new(1989, 9, 10)
 #     end
 #   end
 #
@@ -306,7 +307,7 @@ end
 #    Non-Nil:  7
 #     Unique:  6
 #
-class SimpleDelegator<Delegator
+class SimpleDelegator < Delegator
   # Returns the current object method calls are being delegated to.
   def __getobj__
     unless defined?(@delegate_sd_obj)
@@ -339,12 +340,8 @@ end
 def Delegator.delegating_block(mid) # :nodoc:
   lambda do |*args, &block|
     target = self.__getobj__
-    begin
-      target.__send__(mid, *args, &block)
-    ensure
-      $@.delete_if {|t| /\A#{Regexp.quote(__FILE__)}:#{__LINE__-2}:/o =~ t} if $@
-    end
-  end
+    target.__send__(mid, *args, &block)
+  end.ruby2_keywords
 end
 
 #
@@ -357,6 +354,14 @@ end
 #     end
 #   end
 #
+# or:
+#
+#   MyClass = DelegateClass(ClassToDelegateTo) do    # Step 1
+#     def initialize
+#       super(obj_of_ClassToDelegateTo)              # Step 2
+#     end
+#   end
+#
 # Here's a sample of use from Tempfile which is really a File object with a
 # few special rules about storage location and when the File should be
 # deleted.  That makes for an almost textbook perfect example of how to use
@@ -380,13 +385,15 @@ end
 #     # ...
 #   end
 #
-def DelegateClass(superclass)
+def DelegateClass(superclass, &block)
   klass = Class.new(Delegator)
-  methods = superclass.instance_methods
-  methods -= ::Delegator.public_api
-  methods -= [:to_s,:inspect,:=~,:!~,:===]
+  ignores = [*::Delegator.public_api, :to_s, :inspect, :=~, :!~, :===]
+  protected_instance_methods = superclass.protected_instance_methods
+  protected_instance_methods -= ignores
+  public_instance_methods = superclass.public_instance_methods
+  public_instance_methods -= ignores
   klass.module_eval do
-    def __getobj__  # :nodoc:
+    def __getobj__ # :nodoc:
       unless defined?(@delegate_dc_obj)
         return yield if block_given?
         __raise__ ::ArgumentError, "not delegated"
@@ -397,49 +404,20 @@ def DelegateClass(superclass)
       __raise__ ::ArgumentError, "cannot delegate to self" if self.equal?(obj)
       @delegate_dc_obj = obj
     end
-    methods.each do |method|
+    protected_instance_methods.each do |method|
+      define_method(method, Delegator.delegating_block(method))
+      protected method
+    end
+    public_instance_methods.each do |method|
       define_method(method, Delegator.delegating_block(method))
     end
   end
   klass.define_singleton_method :public_instance_methods do |all=true|
-    super(all) - superclass.protected_instance_methods
+    super(all) | superclass.public_instance_methods
   end
   klass.define_singleton_method :protected_instance_methods do |all=true|
     super(all) | superclass.protected_instance_methods
   end
+  klass.module_eval(&block) if block
   return klass
 end
-
-# :enddoc:
-
-if __FILE__ == $0
-  class ExtArray<DelegateClass(Array)
-    def initialize()
-      super([])
-    end
-  end
-
-  ary = ExtArray.new
-  p ary.class
-  ary.push 25
-  p ary
-  ary.push 42
-  ary.each {|x| p x}
-
-  foo = Object.new
-  def foo.test
-    25
-  end
-  def foo.iter
-    yield self
-  end
-  def foo.error
-    raise 'this is OK'
-  end
-  foo2 = SimpleDelegator.new(foo)
-  p foo2
-  foo2.instance_eval{print "foo\n"}
-  p foo.test == foo2.test       # => true
-  p foo2.iter{[55,true]}        # => true
-  foo2.error                    # raise error!
-end
