@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 #
 # = net/protocol.rb
 #
@@ -11,7 +12,7 @@
 # modify this program under the same terms as Ruby itself,
 # Ruby Distribute License or GNU General Public License.
 #
-# $Id: protocol.rb 46060 2014-05-23 12:36:30Z nobu $
+# $Id$
 #++
 #
 # WARNING: This file is going to remove.
@@ -20,6 +21,7 @@
 
 require 'socket'
 require 'timeout'
+require 'io/wait'
 
 module Net # :nodoc:
 
@@ -32,6 +34,24 @@ module Net # :nodoc:
         end
       End
     end
+
+    def ssl_socket_connect(s, timeout)
+      if timeout
+        while true
+          raise Net::OpenTimeout if timeout <= 0
+          start = Process.clock_gettime Process::CLOCK_MONOTONIC
+          # to_io is required because SSLSocket doesn't have wait_readable yet
+          case s.connect_nonblock(exception: false)
+          when :wait_readable; s.to_io.wait_readable(timeout)
+          when :wait_writable; s.to_io.wait_writable(timeout)
+          else; break
+          end
+          timeout -= Process.clock_gettime(Process::CLOCK_MONOTONIC) - start
+        end
+      else
+        s.connect
+      end
+    end
   end
 
 
@@ -55,20 +75,54 @@ module Net # :nodoc:
   # ReadTimeout, a subclass of Timeout::Error, is raised if a chunk of the
   # response cannot be read within the read_timeout.
 
-  class ReadTimeout            < Timeout::Error; end
+  class ReadTimeout < Timeout::Error
+    def initialize(io = nil)
+      @io = io
+    end
+    attr_reader :io
+
+    def message
+      msg = super
+      if @io
+        msg = "#{msg} with #{@io.inspect}"
+      end
+      msg
+    end
+  end
+
+  ##
+  # WriteTimeout, a subclass of Timeout::Error, is raised if a chunk of the
+  # response cannot be written within the write_timeout.  Not raised on Windows.
+
+  class WriteTimeout < Timeout::Error
+    def initialize(io = nil)
+      @io = io
+    end
+    attr_reader :io
+
+    def message
+      msg = super
+      if @io
+        msg = "#{msg} with #{@io.inspect}"
+      end
+      msg
+    end
+  end
 
 
   class BufferedIO   #:nodoc: internal use only
-    def initialize(io)
+    def initialize(io, read_timeout: 60, write_timeout: 60, continue_timeout: nil, debug_output: nil)
       @io = io
-      @read_timeout = 60
-      @continue_timeout = nil
-      @debug_output = nil
-      @rbuf = ''
+      @read_timeout = read_timeout
+      @write_timeout = write_timeout
+      @continue_timeout = continue_timeout
+      @debug_output = debug_output
+      @rbuf = ''.b
     end
 
     attr_reader :io
     attr_accessor :read_timeout
+    attr_accessor :write_timeout
     attr_accessor :continue_timeout
     attr_accessor :debug_output
 
@@ -94,17 +148,19 @@ module Net # :nodoc:
 
     public
 
-    def read(len, dest = '', ignore_eof = false)
+    def read(len, dest = ''.b, ignore_eof = false)
       LOG "reading #{len} bytes..."
       read_bytes = 0
       begin
         while read_bytes + @rbuf.size < len
-          dest << (s = rbuf_consume(@rbuf.size))
+          s = rbuf_consume(@rbuf.size)
           read_bytes += s.size
+          dest << s
           rbuf_fill
         end
-        dest << (s = rbuf_consume(len - read_bytes))
+        s = rbuf_consume(len - read_bytes)
         read_bytes += s.size
+        dest << s
       rescue EOFError
         raise unless ignore_eof
       end
@@ -112,13 +168,14 @@ module Net # :nodoc:
       dest
     end
 
-    def read_all(dest = '')
+    def read_all(dest = ''.b)
       LOG 'reading all...'
       read_bytes = 0
       begin
         while true
-          dest << (s = rbuf_consume(@rbuf.size))
+          s = rbuf_consume(@rbuf.size)
           read_bytes += s.size
+          dest << s
           rbuf_fill
         end
       rescue EOFError
@@ -149,27 +206,33 @@ module Net # :nodoc:
     BUFSIZE = 1024 * 16
 
     def rbuf_fill
-      begin
-        @rbuf << @io.read_nonblock(BUFSIZE)
-      rescue IO::WaitReadable
-        if IO.select([@io], nil, nil, @read_timeout)
-          retry
-        else
-          raise Net::ReadTimeout
-        end
-      rescue IO::WaitWritable
+      tmp = @rbuf.empty? ? @rbuf : nil
+      case rv = @io.read_nonblock(BUFSIZE, tmp, exception: false)
+      when String
+        return if rv.equal?(tmp)
+        @rbuf << rv
+        rv.clear
+        return
+      when :wait_readable
+        (io = @io.to_io).wait_readable(@read_timeout) or raise Net::ReadTimeout.new(io)
+        # continue looping
+      when :wait_writable
         # OpenSSL::Buffering#read_nonblock may fail with IO::WaitWritable.
         # http://www.openssl.org/support/faq.html#PROG10
-        if IO.select(nil, [@io], nil, @read_timeout)
-          retry
-        else
-          raise Net::ReadTimeout
-        end
-      end
+        (io = @io.to_io).wait_writable(@read_timeout) or raise Net::ReadTimeout.new(io)
+        # continue looping
+      when nil
+        raise EOFError, 'end of file reached'
+      end while true
     end
 
     def rbuf_consume(len)
-      s = @rbuf.slice!(0, len)
+      if len == @rbuf.size
+        s = @rbuf
+        @rbuf = ''.b
+      else
+        s = @rbuf.slice!(0, len)
+      end
       @debug_output << %Q[-> #{s.dump}\n] if @debug_output
       s
     end
@@ -180,9 +243,9 @@ module Net # :nodoc:
 
     public
 
-    def write(str)
+    def write(*strs)
       writing {
-        write0 str
+        write0(*strs)
       }
     end
 
@@ -206,11 +269,34 @@ module Net # :nodoc:
       bytes
     end
 
-    def write0(str)
-      @debug_output << str.dump if @debug_output
-      len = @io.write(str)
-      @written_bytes += len
-      len
+    def write0(*strs)
+      @debug_output << strs.map(&:dump).join if @debug_output
+      orig_written_bytes = @written_bytes
+      strs.each_with_index do |str, i|
+        need_retry = true
+        case len = @io.write_nonblock(str, exception: false)
+        when Integer
+          @written_bytes += len
+          len -= str.bytesize
+          if len == 0
+            if strs.size == i+1
+              return @written_bytes - orig_written_bytes
+            else
+              need_retry = false
+              # next string
+            end
+          elsif len < 0
+            str = str.byteslice(len, -len)
+          else # len > 0
+            need_retry = false
+            # next string
+          end
+          # continue looping
+        when :wait_writable
+          (io = @io.to_io).wait_writable(@write_timeout) or raise Net::WriteTimeout.new(io)
+          # continue looping
+        end while need_retry
+      end
     end
 
     #
@@ -236,7 +322,7 @@ module Net # :nodoc:
 
 
   class InternetMessageIO < BufferedIO   #:nodoc: internal use only
-    def initialize(io)
+    def initialize(*, **)
       super
       @wbuf = nil
     end
@@ -251,7 +337,7 @@ module Net # :nodoc:
       read_bytes = 0
       while (line = readuntil("\r\n")) != ".\r\n"
         read_bytes += line.size
-        yield line.sub(/\A\./, '')
+        yield line.delete_prefix('.')
       end
       LOG_on()
       LOG "read message (#{read_bytes} bytes)"
@@ -313,7 +399,7 @@ module Net # :nodoc:
     end
 
     def using_each_crlf_line
-      @wbuf = ''
+      @wbuf = ''.b
       yield
       if not @wbuf.empty?   # unterminated last line
         write0 dot_stuff(@wbuf.chomp) + "\r\n"
