@@ -2,19 +2,33 @@
 
   main.c -
 
-  $Author: ko1 $
+  $Author$
   created at: Fri Aug 19 13:19:58 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
 
 **********************************************************************/
 
+/*!
+ * \mainpage Developers' documentation for Ruby
+ *
+ * This documentation is produced by applying Doxygen to
+ * <a href="https://github.com/ruby/ruby">Ruby's source code</a>.
+ * It is still under construction (and even not well-maintained).
+ * If you are familiar with Ruby's source code, please improve the doc.
+ */
 #undef RUBY_EXPORT
 #include "ruby.h"
 #include "vm_debug.h"
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
+#if RUBY_DEVEL && !defined RUBY_DEBUG_ENV
+# define RUBY_DEBUG_ENV 1
+#endif
+#if defined RUBY_DEBUG_ENV && !RUBY_DEBUG_ENV
+# undef RUBY_DEBUG_ENV
+#endif
 #ifdef RUBY_DEBUG_ENV
 #include <stdlib.h>
 #endif
