@@ -1,4 +1,4 @@
-require File.expand_path('../../../spec_helper', __FILE__)
+require_relative '../../spec_helper'
 
 describe "Time#wday" do
   it "returns an integer representing the day of the week, 0..6, with Sunday being 0" do
