@@ -1,4 +1,4 @@
-require File.expand_path('../../spec_helper', __FILE__)
+require_relative '../spec_helper'
 
 describe "The BEGIN keyword" do
   before :each do
@@ -15,7 +15,7 @@ describe "The BEGIN keyword" do
   end
 
   it "must appear in a top-level context" do
-    lambda { eval "1.times { BEGIN { 1 } }" }.should raise_error(SyntaxError)
+    -> { eval "1.times { BEGIN { 1 } }" }.should raise_error(SyntaxError)
   end
 
   it "runs first in a given code unit" do
