@@ -1,15 +1,9 @@
-#!/usr/bin/env ruby -w
-# encoding: UTF-8
+# -*- coding: utf-8 -*-
+# frozen_string_literal: false
 
-# tc_table.rb
-#
-#  Created by James Edward Gray II on 2005-10-31.
-#  Copyright 2005 James Edward Gray II. You can redistribute or modify this code
-#  under the terms of Ruby's license.
+require_relative "helper"
 
-require_relative "base"
-
-class TestCSV::Table < TestCSV
+class TestCSVTable < Test::Unit::TestCase
   extend DifferentOFS
 
   def setup
@@ -22,6 +16,8 @@ class TestCSV::Table < TestCSV
     @header_table = CSV::Table.new(
       [CSV::Row.new(%w{A B C}, %w{A B C}, true)] + @rows
     )
+    
+    @header_only_table = CSV::Table.new([], headers: %w{A B C})
   end
 
   def test_initialze
@@ -43,6 +39,11 @@ class TestCSV::Table < TestCSV
     assert_equal(:row, rows.mode)
     assert_equal(@table, rows)
 
+    col_or_row = rows.by_col_or_row
+    assert_equal(:row, rows.mode)
+    assert_equal(:col_or_row, col_or_row.mode)
+    assert_equal(@table, col_or_row)
+
     # destructive mode changing calls
     assert_equal(@table, @table.by_row!)
     assert_equal(:row, @table.mode)
@@ -59,6 +60,17 @@ class TestCSV::Table < TestCSV
     assert_equal Array.new, t.headers
   end
 
+  def test_headers_only
+    assert_equal(%w[A B C], @header_only_table.headers)
+  end
+
+  def test_headers_modified_by_row
+    table = CSV::Table.new([], headers: ["A", "B"])
+    table << ["a", "b"]
+    table.first << {"C" => "c"}
+    assert_equal(["A", "B", "C"], table.headers)
+  end
+
   def test_index
     ##################
     ### Mixed Mode ###
@@ -67,6 +79,9 @@ class TestCSV::Table < TestCSV
     @rows.each_index { |i| assert_equal(@rows[i], @table[i]) }
     assert_equal(nil, @table[100])  # empty row
 
+    # by row with Range
+    assert_equal([@table[1], @table[2]], @table[1..2])
+
     # by col
     @rows.first.headers.each do |header|
       assert_equal(@rows.map { |row| row[header] }, @table[header])
@@ -146,13 +161,13 @@ class TestCSV::Table < TestCSV
                   @table.to_a )
 
     # verify resulting table
-    assert_equal(<<-END_RESULT.gsub(/^\s+/, ""), @table.to_csv)
-    A,B,C,Type,Index
-    1,100,3,data,1
-    4,200,6,data,2
-    10,,12,data,3
-    13,,15,data,
-    END_RESULT
+    assert_equal(<<-CSV, @table.to_csv)
+A,B,C,Type,Index
+1,100,3,data,1
+4,200,6,data,2
+10,,12,data,3
+13,,15,data,
+    CSV
 
     # with headers
     @header_table["Type"] = "data"
@@ -216,6 +231,17 @@ class TestCSV::Table < TestCSV
     # verify that we can chain the call
     assert_equal(@table, @table.each { })
 
+    # without block
+    enum = @table.each
+    assert_instance_of(Enumerator, enum)
+    assert_equal(@table.size, enum.size)
+
+    i = 0
+    enum.each do |row|
+      assert_equal(@rows[i], row)
+      i += 1
+    end
+
     ###################
     ### Column Mode ###
     ###################
@@ -227,6 +253,17 @@ class TestCSV::Table < TestCSV
       assert_equal(@table[header], column)
     end
 
+    # without block
+    enum = @table.each
+    assert_instance_of(Enumerator, enum)
+    assert_equal(@table.headers.size, enum.size)
+
+    headers = @table.headers
+    enum.each do |header, column|
+      assert_equal(headers.shift, header)
+      assert_equal(@table[header], column)
+    end
+
     ############################
     ### One Shot Mode Change ###
     ############################
@@ -237,6 +274,15 @@ class TestCSV::Table < TestCSV
     @table.each { |row| assert_instance_of(CSV::Row, row) }
   end
 
+  def test_each_split
+    yielded_values = []
+    @table.each do |column1, column2, column3|
+      yielded_values << [column1, column2, column3]
+    end
+    assert_equal(@rows.collect(&:to_a),
+                 yielded_values)
+  end
+
   def test_enumerable
     assert_equal( @rows.values_at(0, 2),
                   @table.select { |row| (row["B"] % 2).zero? } )
@@ -253,12 +299,12 @@ class TestCSV::Table < TestCSV
   end
 
   def test_to_csv
-    csv = <<-END_CSV.gsub(/^\s+/, "")
-    A,B,C
-    1,2,3
-    4,5,6
-    7,8,9
-    END_CSV
+    csv = <<-CSV
+A,B,C
+1,2,3
+4,5,6
+7,8,9
+    CSV
 
     # normal conversion
     assert_equal(csv, @table.to_csv)
@@ -286,7 +332,7 @@ class TestCSV::Table < TestCSV
     assert_equal(CSV::Row.new(%w[A B C], [13, 14, 15]), @table[-1])
   end
 
-  def test_delete_mixed
+  def test_delete_mixed_one
     ##################
     ### Mixed Mode ###
     ##################
@@ -297,11 +343,33 @@ class TestCSV::Table < TestCSV
     assert_equal(@rows.map { |row| row["A"] }, @table.delete("A"))
 
     # verify resulting table
-    assert_equal(<<-END_RESULT.gsub(/^\s+/, ""), @table.to_csv)
-    B,C
-    2,3
-    8,9
-    END_RESULT
+    assert_equal(<<-CSV, @table.to_csv)
+B,C
+2,3
+8,9
+    CSV
+  end
+
+  def test_delete_mixed_multiple
+    ##################
+    ### Mixed Mode ###
+    ##################
+    # delete row and col
+    second_row = @rows[1]
+    a_col = @rows.map { |row| row["A"] }
+    a_col_without_second_row = a_col[0..0] + a_col[2..-1]
+    assert_equal([
+                   second_row,
+                   a_col_without_second_row,
+                 ],
+                 @table.delete(1, "A"))
+
+    # verify resulting table
+    assert_equal(<<-CSV, @table.to_csv)
+B,C
+2,3
+8,9
+    CSV
   end
 
   def test_delete_column
@@ -314,12 +382,12 @@ class TestCSV::Table < TestCSV
     assert_equal(@rows.map { |row| row["C"] }, @table.delete("C"))
 
     # verify resulting table
-    assert_equal(<<-END_RESULT.gsub(/^\s+/, ""), @table.to_csv)
-    B
-    2
-    5
-    8
-    END_RESULT
+    assert_equal(<<-CSV, @table.to_csv)
+B
+2
+5
+8
+    CSV
   end
 
   def test_delete_row
@@ -332,11 +400,11 @@ class TestCSV::Table < TestCSV
     assert_raise(TypeError) { @table.delete("C") }
 
     # verify resulting table
-    assert_equal(<<-END_RESULT.gsub(/^\s+/, ""), @table.to_csv)
-    A,B,C
-    1,2,3
-    7,8,9
-    END_RESULT
+    assert_equal(<<-CSV, @table.to_csv)
+A,B,C
+1,2,3
+7,8,9
+    CSV
   end
 
   def test_delete_with_blank_rows
@@ -353,10 +421,28 @@ class TestCSV::Table < TestCSV
     assert_equal(@table, @table.delete_if { |row| (row["B"] % 2).zero? })
 
     # verify resulting table
-    assert_equal(<<-END_RESULT.gsub(/^\s+/, ""), @table.to_csv)
-    A,B,C
-    4,5,6
-    END_RESULT
+    assert_equal(<<-CSV, @table.to_csv)
+A,B,C
+4,5,6
+    CSV
+  end
+
+  def test_delete_if_row_without_block
+    ######################
+    ### Mixed/Row Mode ###
+    ######################
+    enum = @table.delete_if
+    assert_instance_of(Enumerator, enum)
+    assert_equal(@table.size, enum.size)
+
+    # verify that we can chain the call
+    assert_equal(@table, enum.each { |row| (row["B"] % 2).zero? })
+
+    # verify resulting table
+    assert_equal(<<-CSV, @table.to_csv)
+A,B,C
+4,5,6
+    CSV
   end
 
   def test_delete_if_column
@@ -366,12 +452,46 @@ class TestCSV::Table < TestCSV
     @table.by_col!
 
     assert_equal(@table, @table.delete_if { |h, v| h > "A" })
-    assert_equal(<<-END_RESULT.gsub(/^\s+/, ""), @table.to_csv)
-    A
-    1
-    4
-    7
-    END_RESULT
+    assert_equal(<<-CSV, @table.to_csv)
+A
+1
+4
+7
+    CSV
+  end
+
+  def test_delete_if_column_without_block
+    ###################
+    ### Column Mode ###
+    ###################
+    @table.by_col!
+
+    enum = @table.delete_if
+    assert_instance_of(Enumerator, enum)
+    assert_equal(@table.headers.size, enum.size)
+
+    assert_equal(@table, enum.each { |h, v| h > "A" })
+    assert_equal(<<-CSV, @table.to_csv)
+A
+1
+4
+7
+    CSV
+  end
+
+  def test_delete_headers_only
+    ###################
+    ### Column Mode ###
+    ###################
+    @header_only_table.by_col!
+
+    # delete by index
+    assert_equal([], @header_only_table.delete(0))
+    assert_equal(%w[B C], @header_only_table.headers)
+
+    # delete by header
+    assert_equal([], @header_only_table.delete("C"))
+    assert_equal(%w[B], @header_only_table.headers)
   end
 
   def test_values_at
@@ -431,4 +551,70 @@ class TestCSV::Table < TestCSV
                  @table.inspect.encoding],
             "inspect() was not ASCII compatible." )
   end
+
+  def test_dig_mixed
+    # by row
+    assert_equal(@rows[0], @table.dig(0))
+    assert_nil(@table.dig(100))  # empty row
+
+    # by col
+    assert_equal([2, 5, 8], @table.dig("B"))
+    assert_equal([nil] * @rows.size, @table.dig("Z"))  # empty col
+
+    # by row then col
+    assert_equal(2, @table.dig(0, 1))
+    assert_equal(6, @table.dig(1, "C"))
+
+    # by col then row
+    assert_equal(5, @table.dig("B", 1))
+    assert_equal(9, @table.dig("C", 2))
+  end
+
+  def test_dig_by_column
+    @table.by_col!
+
+    assert_equal([2, 5, 8], @table.dig(1))
+    assert_equal([2, 5, 8], @table.dig("B"))
+
+    # by col then row
+    assert_equal(5, @table.dig("B", 1))
+    assert_equal(9, @table.dig("C", 2))
+  end
+
+  def test_dig_by_row
+    @table.by_row!
+
+    assert_equal(@rows[1], @table.dig(1))
+    assert_raise(TypeError) { @table.dig("B") }
+
+    # by row then col
+    assert_equal(2, @table.dig(0, 1))
+    assert_equal(6, @table.dig(1, "C"))
+  end
+
+  def test_dig_cell
+    table = CSV::Table.new([CSV::Row.new(["A"], [["foo", ["bar", ["baz"]]]])])
+
+    # by row, col then cell
+    assert_equal("foo", table.dig(0, "A", 0))
+    assert_equal(["baz"], table.dig(0, "A", 1, 1))
+
+    # by col, row then cell
+    assert_equal("foo", table.dig("A", 0, 0))
+    assert_equal(["baz"], table.dig("A", 0, 1, 1))
+  end
+
+  def test_dig_cell_no_dig
+    table = CSV::Table.new([CSV::Row.new(["A"], ["foo"])])
+
+    # by row, col then cell
+    assert_raise(TypeError) do
+      table.dig(0, "A", 0)
+    end
+
+    # by col, row then cell
+    assert_raise(TypeError) do
+      table.dig("A", 0, 0)
+    end
+  end
 end
