@@ -2,6 +2,10 @@ require_relative 'helper'
 require "reline/history"
 
 class Reline::History::Test < Reline::TestCase
+  def setup
+    Reline.send(:test_mode)
+  end
+
   def test_ancestors
     assert_equal(Reline::History.ancestors.include?(Array), true)
   end
@@ -238,6 +242,26 @@ class Reline::History::Test < Reline::TestCase
     end
   end
 
+  def test_history_size_zero
+    history = history_new(history_size: 0)
+    assert_equal 0, history.size
+    history << 'aa'
+    history << 'bb'
+    assert_equal 0, history.size
+    history.push(*%w{aa bb cc})
+    assert_equal 0, history.size
+  end
+
+  def test_history_size_negative_unlimited
+    history = history_new(history_size: -1)
+    assert_equal 0, history.size
+    history << 'aa'
+    history << 'bb'
+    assert_equal 2, history.size
+    history.push(*%w{aa bb cc})
+    assert_equal 5, history.size
+  end
+
   private
 
   def history_new(history_size: 10)
@@ -268,6 +292,10 @@ class Reline::History::Test < Reline::TestCase
   end
 
   def get_default_internal_encoding
-    return Encoding.default_internal || Encoding.find("locale")
+    if RUBY_PLATFORM =~ /mswin|mingw/
+      Encoding.default_internal || Encoding::UTF_8
+    else
+      Encoding.default_internal || Encoding.find("locale")
+    end
   end
 end
