@@ -8,8 +8,8 @@ class Reline::KeyActor::Emacs::Test < Reline::TestCase
     Reline::HISTORY.instance_variable_set(:@config, @config)
     Reline::HISTORY.clear
     @encoding = (RELINE_TEST_ENCODING rescue Encoding.default_external)
-    @line_editor = Reline::LineEditor.new(@config)
-    @line_editor.reset(@prompt, @encoding)
+    @line_editor = Reline::LineEditor.new(@config, @encoding)
+    @line_editor.reset(@prompt, encoding: @encoding)
   end
 
   def test_ed_insert_one
@@ -1325,6 +1325,68 @@ class Reline::KeyActor::Emacs::Test < Reline::TestCase
     assert_line('foo_ba')
   end
 
+  def test_completion_with_indent
+    @line_editor.completion_proc = proc { |word|
+      %w{
+        foo_foo
+        foo_bar
+        foo_baz
+        qux
+      }.map { |i|
+        i.encode(@encoding)
+      }
+    }
+    input_keys('  fo')
+    assert_byte_pointer_size('  fo')
+    assert_cursor(4)
+    assert_cursor_max(4)
+    assert_line('  fo')
+    assert_equal(nil, @line_editor.instance_variable_get(:@menu_info))
+    input_keys("\C-i", false)
+    assert_byte_pointer_size('  foo_')
+    assert_cursor(6)
+    assert_cursor_max(6)
+    assert_line('  foo_')
+    assert_equal(nil, @line_editor.instance_variable_get(:@menu_info))
+    input_keys("\C-i", false)
+    assert_byte_pointer_size('  foo_')
+    assert_cursor(6)
+    assert_cursor_max(6)
+    assert_line('  foo_')
+    assert_equal(%w{foo_foo foo_bar foo_baz}, @line_editor.instance_variable_get(:@menu_info).list)
+  end
+
+  def test_completion_with_indent_and_completer_quote_characters
+    @line_editor.completion_proc = proc { |word|
+      %w{
+        "".foo_foo
+        "".foo_bar
+        "".foo_baz
+        "".qux
+      }.map { |i|
+        i.encode(@encoding)
+      }
+    }
+    input_keys('  "".fo')
+    assert_byte_pointer_size('  "".fo')
+    assert_cursor(7)
+    assert_cursor_max(7)
+    assert_line('  "".fo')
+    assert_equal(nil, @line_editor.instance_variable_get(:@menu_info))
+    input_keys("\C-i", false)
+    assert_byte_pointer_size('  "".foo_')
+    assert_cursor(9)
+    assert_cursor_max(9)
+    assert_line('  "".foo_')
+    assert_equal(nil, @line_editor.instance_variable_get(:@menu_info))
+    input_keys("\C-i", false)
+    assert_byte_pointer_size('  "".foo_')
+    assert_cursor(9)
+    assert_cursor_max(9)
+    assert_line('  "".foo_')
+    assert_equal(%w{"".foo_foo "".foo_bar "".foo_baz}, @line_editor.instance_variable_get(:@menu_info).list)
+  end
+
   def test_completion_with_perfect_match
     @line_editor.completion_proc = proc { |word|
       %w{
@@ -1563,7 +1625,7 @@ class Reline::KeyActor::Emacs::Test < Reline::TestCase
     assert_line('')
   end
 
-  def test_ed_search_prev_history
+  def test_vi_search_prev
     Reline::HISTORY.concat(%w{abc 123 AAA})
     assert_line('')
     assert_byte_pointer_size('')
@@ -1834,6 +1896,165 @@ class Reline::KeyActor::Emacs::Test < Reline::TestCase
     assert_equal([0, 0], @line_editor.instance_variable_get(:@mark_pointer))
   end
 
+  def test_modify_lines_with_wrong_rs
+    verbose, $VERBOSE = $VERBOSE, nil
+    original_global_slash = $/
+    $/ = 'b'
+    $VERBOSE = verbose
+    @line_editor.output_modifier_proc = proc { |output| Reline::Unicode.escape_for_print(output) }
+    input_keys("abcdef\n")
+    result = @line_editor.__send__(:modify_lines, @line_editor.whole_lines)
+    $/ = nil
+    assert_equal(['abcdef'], result)
+  ensure
+    $VERBOSE = nil
+    $/ = original_global_slash
+    $VERBOSE = verbose
+  end
+
+  def test_ed_search_prev_history
+    Reline::HISTORY.concat([
+      '12356', # old
+      '12aaa',
+      '12345' # new
+    ])
+    input_keys('123')
+    # The ed_search_prev_history doesn't have default binding
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('123')
+    assert_cursor(3)
+    assert_cursor_max(5)
+    assert_line('12345')
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('123')
+    assert_cursor(3)
+    assert_cursor_max(5)
+    assert_line('12356')
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('123')
+    assert_cursor(3)
+    assert_cursor_max(5)
+    assert_line('12356')
+  end
+
+  def test_ed_search_prev_history_with_empty
+    Reline::HISTORY.concat([
+      '12356', # old
+      '12aaa',
+      '12345' # new
+    ])
+    # The ed_search_prev_history doesn't have default binding
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('')
+    assert_cursor(0)
+    assert_cursor_max(5)
+    assert_line('12345')
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('')
+    assert_cursor(0)
+    assert_cursor_max(5)
+    assert_line('12aaa')
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('')
+    assert_cursor(0)
+    assert_cursor_max(5)
+    assert_line('12356')
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('')
+    assert_cursor(0)
+    assert_cursor_max(5)
+    assert_line('12356')
+  end
+
+  def test_ed_search_prev_history_without_match
+    Reline::HISTORY.concat([
+      '12356', # old
+      '12aaa',
+      '12345' # new
+    ])
+    input_keys('ABC')
+    # The ed_search_prev_history doesn't have default binding
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('ABC')
+    assert_cursor(3)
+    assert_cursor_max(3)
+    assert_line('ABC')
+  end
+
+  def test_ed_search_next_history
+    Reline::HISTORY.concat([
+      '12356', # old
+      '12aaa',
+      '12345' # new
+    ])
+    input_keys('123')
+    # The ed_search_prev_history and ed_search_next_history doesn't have default binding
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('123')
+    assert_cursor(3)
+    assert_cursor_max(5)
+    assert_line('12345')
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('123')
+    assert_cursor(3)
+    assert_cursor_max(5)
+    assert_line('12356')
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('123')
+    assert_cursor(3)
+    assert_cursor_max(5)
+    assert_line('12356')
+    @line_editor.__send__(:ed_search_next_history, "\C-n".ord)
+    assert_byte_pointer_size('123')
+    assert_cursor(3)
+    assert_cursor_max(5)
+    assert_line('12345')
+    @line_editor.__send__(:ed_search_next_history, "\C-n".ord)
+    assert_byte_pointer_size('123')
+    assert_cursor(3)
+    assert_cursor_max(5)
+    assert_line('12345')
+  end
+
+  def test_ed_search_next_history_with_empty
+    Reline::HISTORY.concat([
+      '12356', # old
+      '12aaa',
+      '12345' # new
+    ])
+    # The ed_search_prev_history and ed_search_next_history doesn't have default binding
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('')
+    assert_cursor(0)
+    assert_cursor_max(5)
+    assert_line('12345')
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('')
+    assert_cursor(0)
+    assert_cursor_max(5)
+    assert_line('12aaa')
+    @line_editor.__send__(:ed_search_prev_history, "\C-p".ord)
+    assert_byte_pointer_size('')
+    assert_cursor(0)
+    assert_cursor_max(5)
+    assert_line('12356')
+    @line_editor.__send__(:ed_search_next_history, "\C-n".ord)
+    assert_byte_pointer_size('')
+    assert_cursor(0)
+    assert_cursor_max(5)
+    assert_line('12aaa')
+    @line_editor.__send__(:ed_search_next_history, "\C-n".ord)
+    assert_byte_pointer_size('')
+    assert_cursor(0)
+    assert_cursor_max(5)
+    assert_line('12345')
+    @line_editor.__send__(:ed_search_next_history, "\C-n".ord)
+    assert_byte_pointer_size('')
+    assert_cursor(0)
+    assert_cursor_max(0)
+    assert_line('')
+  end
+
 =begin # TODO: move KeyStroke instance from Reline to LineEditor
   def test_key_delete
     input_keys('ab')
