@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 begin
   require_relative 'dummyparser'
   require 'test/unit'
@@ -24,6 +25,14 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     parse(str, :compile_error) {|e, msg| return msg}
   end
 
+  def warning(str)
+    parse(str, :warning) {|e, *args| return args}
+  end
+
+  def warn(str)
+    parse(str, :warn) {|e, *args| return args}
+  end
+
   def test_program
     thru_program = false
     assert_equal '[void()]', parse('', :on_program) {thru_program = true}
@@ -49,6 +58,9 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     assert_equal '[assign(var_field(a),ref(a))]', parse('a=a')
     assert_equal '[ref(nil)]', parse('nil')
     assert_equal '[ref(true)]', parse('true')
+    assert_equal '[vcall(_0)]', parse('_0')
+    assert_equal '[vcall(_1)]', parse('_1')
+    assert_include parse('proc{_1}'), '[ref(_1)]'
   end
 
   def test_vcall
@@ -119,6 +131,20 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     assert_equal true, thru_args_new
   end
 
+  def test_args_forward
+    [
+      'def m(...) n(...) end',
+      'def m(...) end',
+      'def m(a, ...) n(1, ...) end',
+      'def m(...) n(1, ...) end',
+      'def m(a, ...) n(...) end'
+    ].each do |code|
+      thru_args_forward = false
+      parse(code, :on_args_forward) {thru_args_forward = true}
+      assert_equal true, thru_args_forward, "no args_forward for: #{code}"
+    end
+  end
+
   def test_arg_paren
     # FIXME
   end
@@ -164,8 +190,14 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
 
   def test_operator_ambiguous
     thru_operator_ambiguous = false
-    parse('a=1; a %[]', :on_operator_ambiguous) {thru_operator_ambiguous = true}
+    token = syntax = nil
+    parse('a=1; a %[]', :on_operator_ambiguous) {|*a|
+      thru_operator_ambiguous = true
+      _, token, syntax = *a
+    }
     assert_equal true, thru_operator_ambiguous
+    assert_equal :%, token
+    assert_equal "string literal", syntax
   end
 
   def test_array   # array literal
@@ -179,38 +211,64 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
   end
 
   def test_assign_error
-    # for test_coverage
+    thru_assign_error = false
+    result = parse('self = 1', :on_assign_error) {thru_assign_error = true}
+    assert_equal true, thru_assign_error
+    assert_equal '[assign(assign_error(var_field(self)),1)]', result
   end
 
   def test_assign_error_backref
     thru_assign_error = false
-    parse('$` = 1', :on_assign_error) {thru_assign_error = true}
+    result =
+      parse('$` = 1', :on_assign_error) {thru_assign_error = true}
     assert_equal true, thru_assign_error
+    assert_equal '[assign(assign_error(var_field($`)),1)]', result
+
     thru_assign_error = false
-    parse('$`, _ = 1', :on_assign_error) {thru_assign_error = true}
+    result =
+      parse('$`, _ = 1', :on_assign_error) {thru_assign_error = true}
     assert_equal true, thru_assign_error
+    assert_equal '[massign([assign_error(var_field($`)),var_field(_)],1)]', result
   end
 
   def test_assign_error_const_qualified
     thru_assign_error = false
-    parse('self::X = 1', :on_assign_error) {thru_assign_error = true}
+    result =
+      parse('self::X = 1', :on_assign_error) {thru_assign_error = true}
     assert_equal false, thru_assign_error
-    parse("def m\n self::X = 1\nend", :on_assign_error) {thru_assign_error = true}
-    assert_equal true, thru_assign_error
+    assert_equal "[assign(const_path_field(ref(self),X),1)]", result
+
     thru_assign_error = false
-    parse("def m\n self::X, a = 1, 2\nend", :on_assign_error) {thru_assign_error = true}
+    result =
+      parse("def m\n self::X = 1\nend", :on_assign_error) {thru_assign_error = true}
     assert_equal true, thru_assign_error
+    assert_include result, "assign_error(const_path_field(ref(self),X))"
+
+    thru_assign_error = false
+    result =
+      parse("def m\n self::X, a = 1, 2\nend", :on_assign_error) {thru_assign_error = true}
+    assert_equal true, thru_assign_error
+    assert_include result, "assign_error(const_path_field(ref(self),X))"
   end
 
   def test_assign_error_const
     thru_assign_error = false
-    parse('X = 1', :on_assign_error) {thru_assign_error = true}
+    result = parse('X = 1', :on_assign_error) {thru_assign_error = true}
     assert_equal false, thru_assign_error
-    parse("def m\n X = 1\nend", :on_assign_error) {thru_assign_error = true}
-    assert_equal true, thru_assign_error
+    assert_equal "[assign(var_field(X),1)]", result
+
     thru_assign_error = false
-    parse("def m\n X, a = 1, 2\nend", :on_assign_error) {thru_assign_error = true}
+    result = parse('X, a = 1, 2', :on_assign_error) {thru_assign_error = true}
+    assert_equal false, thru_assign_error
+    assert_include result, "massign([var_field(X),var_field(a)],"
+
+    result = parse("def m\n X = 1\nend", :on_assign_error) {thru_assign_error = true}
     assert_equal true, thru_assign_error
+    assert_include result, "assign_error(var_field(X))"
+    thru_assign_error = false
+    result = parse("def m\n X, a = 1, 2\nend", :on_assign_error) {thru_assign_error = true}
+    assert_equal true, thru_assign_error
+    assert_include result, "assign_error(var_field(X))"
   end
 
   def test_assign_error_const_toplevel
@@ -346,12 +404,51 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     }
     assert_equal true, thru_call
     assert_equal "[call(ref(self),.,foo)]", tree
+
+    thru_call = false
+    assert_nothing_raised {
+      tree = parse("self.foo()", :on_call) {thru_call = true}
+    }
+    assert_equal true, thru_call
+    assert_equal "[call(ref(self),.,foo,[])]", tree
+
     thru_call = false
     assert_nothing_raised(bug2233) {
       tree = parse("foo.()", :on_call) {thru_call = true}
     }
     assert_equal true, thru_call
     assert_equal "[call(vcall(foo),.,call,[])]", tree
+
+    thru_call = false
+    assert_nothing_raised {
+      tree = parse("self::foo", :on_call) {thru_call = true}
+    }
+    assert_equal true, thru_call
+    assert_equal "[call(ref(self),::,foo)]", tree
+
+    thru_call = false
+    assert_nothing_raised {
+      tree = parse("self::foo()", :on_call) {thru_call = true}
+    }
+    assert_equal true, thru_call
+    assert_equal "[call(ref(self),::,foo,[])]", tree
+
+    thru_call = false
+    assert_nothing_raised(bug2233) {
+      tree = parse("foo::()", :on_call) {thru_call = true}
+    }
+    assert_equal true, thru_call
+    assert_equal "[call(vcall(foo),::,call,[])]", tree
+
+    thru_call = false
+    tree = parse("self&.foo", :on_call) {thru_call = true}
+    assert_equal true, thru_call
+    assert_equal "[call(ref(self),&.,foo)]", tree
+
+    thru_call = false
+    tree = parse("self&.foo()", :on_call) {thru_call = true}
+    assert_equal true, thru_call
+    assert_equal "[call(ref(self),&.,foo,[])]", tree
   end
 
   def test_excessed_comma
@@ -384,6 +481,29 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     assert_equal("heredoc1\nheredoc2\n", heredoc, bug1921)
   end
 
+  def test_heredoc_dedent
+    thru_heredoc_dedent = false
+    str = width = nil
+    tree = parse("<""<~EOS\n heredoc\nEOS\n", :on_heredoc_dedent) {|e, s, w|
+      thru_heredoc_dedent = true
+      str = s
+      width = w
+    }
+    assert_equal true, thru_heredoc_dedent
+    assert_match(/string_content\(\), heredoc\n/, tree)
+    assert_equal(" heredoc\n", str.children[1])
+    assert_equal(1, width)
+  end
+
+  def test_unterminated_heredoc
+    assert_match("can't find string \"a\" anywhere before EOF", compile_error("<<a"))
+    assert_match("can't find string \"a\" anywhere before EOF", compile_error('<<"a"'))
+    assert_match("can't find string \"a\" anywhere before EOF", compile_error("<<'a'"))
+    msg = nil
+    parse('<<"', :on_parse_error) {|_, e| msg = e}
+    assert_equal("unterminated here document identifier", msg)
+  end
+
   def test_massign
     thru_massign = false
     parse("a, b = 1, 2", :on_massign) {thru_massign = true}
@@ -403,23 +523,46 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     thru_mlhs_add_star = false
     tree = parse("a, *b = 1, 2", :on_mlhs_add_star) {thru_mlhs_add_star = true}
     assert_equal true, thru_mlhs_add_star
-    assert_match(/mlhs_add_star\(mlhs_add\(mlhs_new\(\),a\),b\)/, tree)
+    assert_include(tree, "massign([var_field(a),*var_field(b)]")
     thru_mlhs_add_star = false
     tree = parse("a, *b, c = 1, 2", :on_mlhs_add_star) {thru_mlhs_add_star = true}
     assert_equal true, thru_mlhs_add_star
-    assert_match(/mlhs_add\(mlhs_add_star\(mlhs_add\(mlhs_new\(\),a\),b\),mlhs_add\(mlhs_new\(\),c\)\)/, tree, bug2232)
+    assert_include(tree, "massign([var_field(a),*var_field(b),var_field(c)]", bug2232)
     thru_mlhs_add_star = false
     tree = parse("a, *, c = 1, 2", :on_mlhs_add_star) {thru_mlhs_add_star = true}
     assert_equal true, thru_mlhs_add_star
-    assert_match(/mlhs_add\(mlhs_add_star\(mlhs_add\(mlhs_new\(\),a\)\),mlhs_add\(mlhs_new\(\),c\)\)/, tree, bug4364)
+    assert_include(tree, "massign([var_field(a),*,var_field(c)]", bug4364)
     thru_mlhs_add_star = false
     tree = parse("*b, c = 1, 2", :on_mlhs_add_star) {thru_mlhs_add_star = true}
     assert_equal true, thru_mlhs_add_star
-    assert_match(/mlhs_add\(mlhs_add_star\(mlhs_new\(\),b\),mlhs_add\(mlhs_new\(\),c\)\)/, tree, bug4364)
+    assert_include(tree, "massign([*var_field(b),var_field(c)]", bug4364)
     thru_mlhs_add_star = false
     tree = parse("*, c = 1, 2", :on_mlhs_add_star) {thru_mlhs_add_star = true}
     assert_equal true, thru_mlhs_add_star
-    assert_match(/mlhs_add\(mlhs_add_star\(mlhs_new\(\)\),mlhs_add\(mlhs_new\(\),c\)\)/, tree, bug4364)
+    assert_include(tree, "massign([*,var_field(c)],", bug4364)
+  end
+
+  def test_mlhs_add_post
+    thru_mlhs_add_post = false
+    tree = parse("a, *b = 1, 2", :on_mlhs_add_post) {thru_mlhs_add_post = true}
+    assert_equal false, thru_mlhs_add_post
+    assert_include(tree, "massign([var_field(a),*var_field(b)],")
+    thru_mlhs_add_post = false
+    tree = parse("a, *b, c = 1, 2", :on_mlhs_add_post) {thru_mlhs_add_post = true}
+    assert_equal true, thru_mlhs_add_post
+    assert_include(tree, "massign([var_field(a),*var_field(b),var_field(c)],")
+    thru_mlhs_add_post = false
+    tree = parse("a, *, c = 1, 2", :on_mlhs_add_post) {thru_mlhs_add_post = true}
+    assert_equal true, thru_mlhs_add_post
+    assert_include(tree, "massign([var_field(a),*,var_field(c)],")
+    thru_mlhs_add_post = false
+    tree = parse("*b, c = 1, 2", :on_mlhs_add_post) {thru_mlhs_add_post = true}
+    assert_equal true, thru_mlhs_add_post
+    assert_include(tree, "massign([*var_field(b),var_field(c)],")
+    thru_mlhs_add_post = false
+    tree = parse("*, c = 1, 2", :on_mlhs_add_post) {thru_mlhs_add_post = true}
+    assert_equal true, thru_mlhs_add_post
+    assert_include(tree, "massign([*,var_field(c)],")
   end
 
   def test_mlhs_new
@@ -517,8 +660,13 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
 
   def test_defs
     thru_defs = false
-    parse('def foo.bar; end', :on_defs) {thru_defs = true}
+    tree = parse('def foo.bar; end', :on_defs) {thru_defs = true}
     assert_equal true, thru_defs
+    assert_equal("[defs(vcall(foo),.,bar,[],bodystmt([void()]))]", tree)
+
+    thru_parse_error = false
+    tree = parse('def foo&.bar; end', :on_parse_error) {thru_parse_error = true}
+    assert_equal(true, thru_parse_error)
   end
 
   def test_do_block
@@ -612,6 +760,12 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     assert_equal true, thru_ifop
   end
 
+  def test_ignored_nl
+    ignored_nl = []
+    parse("foo # comment\n...\n", :on_ignored_nl) {|_, a| ignored_nl << a}
+    assert_equal ["\n"], ignored_nl
+  end
+
   def test_lambda
     thru_lambda = false
     parse('->{}', :on_lambda) {thru_lambda = true}
@@ -686,44 +840,67 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
 
   def test_opassign
     thru_opassign = false
-    parse('a += b', :on_opassign) {thru_opassign = true}
+    tree = parse('a += b', :on_opassign) {thru_opassign = true}
+    assert_equal true, thru_opassign
+    assert_equal "[opassign(var_field(a),+=,vcall(b))]", tree
+    thru_opassign = false
+    tree = parse('a -= b', :on_opassign) {thru_opassign = true}
+    assert_equal true, thru_opassign
+    assert_equal "[opassign(var_field(a),-=,vcall(b))]", tree
+    thru_opassign = false
+    tree = parse('a *= b', :on_opassign) {thru_opassign = true}
+    assert_equal true, thru_opassign
+    assert_equal "[opassign(var_field(a),*=,vcall(b))]", tree
+    thru_opassign = false
+    tree = parse('a /= b', :on_opassign) {thru_opassign = true}
+    assert_equal true, thru_opassign
+    assert_equal "[opassign(var_field(a),/=,vcall(b))]", tree
+    thru_opassign = false
+    tree = parse('a %= b', :on_opassign) {thru_opassign = true}
+    assert_equal true, thru_opassign
+    assert_equal "[opassign(var_field(a),%=,vcall(b))]", tree
+    thru_opassign = false
+    tree = parse('a **= b', :on_opassign) {thru_opassign = true}
+    assert_equal true, thru_opassign
+    assert_equal "[opassign(var_field(a),**=,vcall(b))]", tree
+    thru_opassign = false
+    tree = parse('a &= b', :on_opassign) {thru_opassign = true}
+    assert_equal true, thru_opassign
+    assert_equal "[opassign(var_field(a),&=,vcall(b))]", tree
+    thru_opassign = false
+    tree = parse('a |= b', :on_opassign) {thru_opassign = true}
+    assert_equal "[opassign(var_field(a),|=,vcall(b))]", tree
     assert_equal true, thru_opassign
     thru_opassign = false
-    parse('a -= b', :on_opassign) {thru_opassign = true}
+    tree = parse('a <<= b', :on_opassign) {thru_opassign = true}
     assert_equal true, thru_opassign
+    assert_equal "[opassign(var_field(a),<<=,vcall(b))]", tree
     thru_opassign = false
-    parse('a *= b', :on_opassign) {thru_opassign = true}
+    tree = parse('a >>= b', :on_opassign) {thru_opassign = true}
     assert_equal true, thru_opassign
+    assert_equal "[opassign(var_field(a),>>=,vcall(b))]", tree
     thru_opassign = false
-    parse('a /= b', :on_opassign) {thru_opassign = true}
+    tree = parse('a &&= b', :on_opassign) {thru_opassign = true}
     assert_equal true, thru_opassign
+    assert_equal "[opassign(var_field(a),&&=,vcall(b))]", tree
     thru_opassign = false
-    parse('a %= b', :on_opassign) {thru_opassign = true}
+    tree = parse('a ||= b', :on_opassign) {thru_opassign = true}
     assert_equal true, thru_opassign
+    assert_equal "[opassign(var_field(a),||=,vcall(b))]", tree
     thru_opassign = false
-    parse('a **= b', :on_opassign) {thru_opassign = true}
+    tree = parse('a::X ||= c 1', :on_opassign) {thru_opassign = true}
     assert_equal true, thru_opassign
+    assert_equal "[opassign(const_path_field(vcall(a),X),||=,command(c,[1]))]", tree
+
     thru_opassign = false
-    parse('a &= b', :on_opassign) {thru_opassign = true}
+    tree = parse("self.foo += 1", :on_opassign) {thru_opassign = true}
     assert_equal true, thru_opassign
+    assert_equal "[opassign(field(ref(self),.,foo),+=,1)]", tree
+
     thru_opassign = false
-    parse('a |= b', :on_opassign) {thru_opassign = true}
-    assert_equal true, thru_opassign
-    thru_opassign = false
-    parse('a <<= b', :on_opassign) {thru_opassign = true}
-    assert_equal true, thru_opassign
-    thru_opassign = false
-    parse('a >>= b', :on_opassign) {thru_opassign = true}
-    assert_equal true, thru_opassign
-    thru_opassign = false
-    parse('a &&= b', :on_opassign) {thru_opassign = true}
-    assert_equal true, thru_opassign
-    thru_opassign = false
-    parse('a ||= b', :on_opassign) {thru_opassign = true}
-    assert_equal true, thru_opassign
-    thru_opassign = false
-    parse('a::X ||= c 1', :on_opassign) {thru_opassign = true}
+    tree = parse("self&.foo += 1", :on_opassign) {thru_opassign = true}
     assert_equal true, thru_opassign
+    assert_equal "[opassign(field(ref(self),&.,foo),+=,1)]", tree
   end
 
   def test_opassign_error
@@ -775,7 +952,89 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     thru_params = false
     parse('a {|**x|}', :on_params) {|_, *v| thru_params = true; arg = v}
     assert_equal true, thru_params
-    assert_equal [nil, nil, nil, nil, nil, "x", nil], arg
+    assert_equal [nil, nil, nil, nil, nil, "**x", nil], arg
+    thru_params = false
+    parse('a {|**nil|}', :on_params) {|_, *v| thru_params = true; arg = v}
+    assert_equal true, thru_params
+    assert_equal [nil, nil, nil, nil, nil, :nil, nil], arg
+  end
+
+  def test_params_mlhs
+    thru_mlhs = false
+    tree = parse("proc {|(a, b)|}", :on_mlhs_paren) {thru_mlhs = true}
+    assert_equal true, thru_mlhs
+    assert_include(tree, "[mlhs([a,b])]")
+  end
+
+  def test_params_mlhs_add
+    thru_mlhs_add = false
+    tree = parse("proc {|(a, b)|}", :on_mlhs_add) {thru_mlhs_add = true}
+    assert_equal true, thru_mlhs_add
+    assert_include(tree, "[mlhs([a,b])]")
+  end
+
+  def test_params_mlhs_add_star
+    thru_mlhs_add_star = false
+    tree = parse("proc {|(a, *b)|}", :on_mlhs_add_star) {thru_mlhs_add_star = true}
+    assert_equal true, thru_mlhs_add_star
+    assert_include(tree, "[mlhs([a,*b])]")
+    thru_mlhs_add_star = false
+    tree = parse("proc {|(a, *b, c)|}", :on_mlhs_add_star) {thru_mlhs_add_star = true}
+    assert_equal true, thru_mlhs_add_star
+    assert_include(tree, "[mlhs([a,*b,c])]")
+    thru_mlhs_add_star = false
+    tree = parse("proc {|(a, *, c)|}", :on_mlhs_add_star) {thru_mlhs_add_star = true}
+    assert_equal true, thru_mlhs_add_star
+    assert_include(tree, "[mlhs([a,*,c])]")
+    thru_mlhs_add_star = false
+    tree = parse("proc {|(*b, c)|}", :on_mlhs_add_star) {thru_mlhs_add_star = true}
+    assert_equal true, thru_mlhs_add_star
+    assert_include(tree, "[mlhs([*b,c])]")
+    thru_mlhs_add_star = false
+    tree = parse("proc {|(*b)|}", :on_mlhs_add_star) {thru_mlhs_add_star = true}
+    assert_equal true, thru_mlhs_add_star
+    assert_include(tree, "[mlhs([*b])]")
+  end
+
+  def test_params_mlhs_add_post
+    thru_mlhs_add_post = false
+    tree = parse("proc {|(a, *b)|}", :on_mlhs_add_post) {thru_mlhs_add_post = true}
+    assert_equal false, thru_mlhs_add_post
+    assert_include(tree, "mlhs([a,*b])")
+    thru_mlhs_add_post = false
+    tree = parse("proc {|(a, *b, c)|}", :on_mlhs_add_post) {thru_mlhs_add_post = true}
+    assert_equal true, thru_mlhs_add_post
+    assert_include(tree, "mlhs([a,*b,c])")
+    thru_mlhs_add_post = false
+    tree = parse("proc {|(a, *, c)|}", :on_mlhs_add_post) {thru_mlhs_add_post = true}
+    assert_equal true, thru_mlhs_add_post
+    assert_include(tree, "mlhs([a,*,c])")
+    thru_mlhs_add_post = false
+    tree = parse("proc {|(*b, c)|}", :on_mlhs_add_post) {thru_mlhs_add_post = true}
+    assert_equal true, thru_mlhs_add_post
+    assert_include(tree, "mlhs([*b,c])")
+    thru_mlhs_add_post = false
+    tree = parse("proc {|(*, c)|}", :on_mlhs_add_post) {thru_mlhs_add_post = true}
+    assert_equal true, thru_mlhs_add_post
+    assert_include(tree, "mlhs([*,c])")
+  end
+
+  def test_params_mlhs_new
+    thru_mlhs_new = false
+    tree = parse("proc {|(a, b)|}", :on_mlhs_new) {thru_mlhs_new = true}
+    assert_equal true, thru_mlhs_new
+    assert_include(tree, "[mlhs([a,b])]")
+  end
+
+  def test_params_mlhs_paren
+    thru_mlhs_paren = 0
+    tree = parse("proc {|(a, b)|}", :on_mlhs_paren) {thru_mlhs_paren += 1}
+    assert_equal 1, thru_mlhs_paren
+    assert_include(tree, "[mlhs([a,b])]")
+    thru_mlhs_paren = 0
+    tree = parse("proc {|((a, b))|}", :on_mlhs_paren) {thru_mlhs_paren += 1}
+    assert_equal 2, thru_mlhs_paren
+    assert_include(tree, "[mlhs([a,b])]")
   end
 
   def test_paren
@@ -792,20 +1051,35 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
 
   def test_qwords_add
     thru_qwords_add = false
-    parse('%w[a]', :on_qwords_add) {thru_qwords_add = true}
+    tree = parse('%w[a]', :on_qwords_add) {thru_qwords_add = true}
     assert_equal true, thru_qwords_add
+    assert_equal '[array([a])]', tree
+    thru_qwords_add = false
+    tree = parse('%w[ a ]', :on_qwords_add) {thru_qwords_add = true}
+    assert_equal true, thru_qwords_add
+    assert_equal '[array([a])]', tree
   end
 
   def test_qsymbols_add
     thru_qsymbols_add = false
-    parse('%i[a]', :on_qsymbols_add) {thru_qsymbols_add = true}
+    tree = parse('%i[a]', :on_qsymbols_add) {thru_qsymbols_add = true}
     assert_equal true, thru_qsymbols_add
+    assert_equal '[array([:a])]', tree
+    thru_qsymbols_add = false
+    tree = parse('%i[ a ]', :on_qsymbols_add) {thru_qsymbols_add = true}
+    assert_equal true, thru_qsymbols_add
+    assert_equal '[array([:a])]', tree
   end
 
   def test_symbols_add
     thru_symbols_add = false
-    parse('%I[a]', :on_symbols_add) {thru_symbols_add = true}
+    tree = parse('%I[a]', :on_symbols_add) {thru_symbols_add = true}
     assert_equal true, thru_symbols_add
+    assert_equal '[array([:a])]', tree
+    thru_symbols_add = false
+    tree = parse('%I[ a ]', :on_symbols_add) {thru_symbols_add = true}
+    assert_equal true, thru_symbols_add
+    assert_equal '[array([:a])]', tree
   end
 
   def test_qwords_new
@@ -883,6 +1157,21 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     assert_equal true, thru_rest_param
   end
 
+  def test_kwrest_param
+    thru_kwrest = false
+    parse('def a(**) end', :on_kwrest_param) {|n, val| thru_kwrest = val}
+    assert_equal nil, thru_kwrest
+    thru_kwrest = false
+    parse('def a(**x) end', :on_kwrest_param) {|n, val| thru_kwrest = val}
+    assert_equal "x", thru_kwrest
+  end
+
+  def test_nokw_param
+    thru_nokw = false
+    parse('def a(**nil) end', :on_nokw_param) {|n, val| thru_nokw = val}
+    assert_equal nil, thru_nokw
+  end
+
   def test_retry
     thru_retry = false
     parse('retry', :on_retry) {thru_retry = true}
@@ -1146,8 +1435,13 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
 
   def test_words_add
     thru_words_add = false
-    parse('%W[a]', :on_words_add) {thru_words_add = true}
+    tree = parse('%W[a]', :on_words_add) {thru_words_add = true}
     assert_equal true, thru_words_add
+    assert_equal '[array([a])]', tree
+    thru_words_add = false
+    tree = parse('%W[ a ]', :on_words_add) {thru_words_add = true}
+    assert_equal true, thru_words_add
+    assert_equal '[array([a])]', tree
   end
 
   def test_words_new
@@ -1206,8 +1500,12 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
   def test_block_variables
     assert_equal("[fcall(proc,[],&block([],[void()]))]", parse("proc{|;y|}"))
     if defined?(Process::RLIMIT_AS)
-      assert_in_out_err(["-I#{File.dirname(__FILE__)}", "-rdummyparser"],
-                        'Process.setrlimit(Process::RLIMIT_AS,100*1024*1024); puts DummyParser.new("proc{|;y|!y}").parse',
+      dir = File.dirname(__FILE__)
+      as = 100 * 1024 * 1024 # 100MB
+      as *= 2 if RubyVM::MJIT.enabled? # space for compiler
+      assert_in_out_err(%W(-I#{dir} -rdummyparser),
+                        "Process.setrlimit(Process::RLIMIT_AS,#{as}); "\
+                        "puts DummyParser.new('proc{|;y|!y}').parse",
                         ["[fcall(proc,[],&block([],[unary(!,ref(y))]))]"], [], '[ruby-dev:39423]')
     end
   end
@@ -1217,16 +1515,55 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
   end
 
   def test_invalid_instance_variable_name
-    assert_equal("`@1' is not allowed as an instance variable name", compile_error('@1'))
-    assert_equal("`@%' is not allowed as an instance variable name", compile_error('@%'))
+    assert_equal("`@1' is not allowed as an instance variable name", compile_error('proc{@1}'))
+    assert_equal("`@' without identifiers is not allowed as an instance variable name", compile_error('@%'))
+    assert_equal("`@' without identifiers is not allowed as an instance variable name", compile_error('@'))
   end
 
   def test_invalid_class_variable_name
     assert_equal("`@@1' is not allowed as a class variable name", compile_error('@@1'))
-    assert_equal("`@@%' is not allowed as a class variable name", compile_error('@@%'))
+    assert_equal("`@@' without identifiers is not allowed as a class variable name", compile_error('@@%'))
+    assert_equal("`@@' without identifiers is not allowed as a class variable name", compile_error('@@'))
   end
 
   def test_invalid_global_variable_name
     assert_equal("`$%' is not allowed as a global variable name", compile_error('$%'))
+    assert_equal("`$' without identifiers is not allowed as a global variable name", compile_error('$'))
+  end
+
+  def test_warning_ignored_magic_comment
+    fmt, *args = warning("1; #-*- frozen-string-literal: true -*-")
+    assert_match(/ignored after any tokens/, fmt)
+    assert_equal("frozen_string_literal", args[0])
+  end
+
+  def test_warn_cr_in_middle
+    fmt = nil
+    assert_warn("") {fmt, = warn("\r;")}
+    assert_match(/encountered/, fmt)
+  end
+
+  def test_warn_mismatched_indentations
+    fmt, tokend, tokbeg, line = assert_warning("") {break warn("if true\n  end\n")}
+    assert_match(/mismatched indentations/, fmt)
+    assert_equal(["if", "end", 1], [tokbeg, tokend, line])
+  end
+
+  def test_in
+    thru_in = false
+    parse('case 0; in 0; end', :on_in) {thru_in = true}
+    assert_equal true, thru_in
+  end
+
+  def test_aryptn
+    thru_aryptn = false
+    parse('case 0; in [0]; end', :on_aryptn) {thru_aryptn = true}
+    assert_equal true, thru_aryptn
+  end
+
+  def test_hshptn
+    thru_hshptn = false
+    parse('case 0; in {a:}; end', :on_hshptn) {thru_hshptn = true}
+    assert_equal true, thru_hshptn
   end
 end if ripper_test
