@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 require 'test/unit'
 
 class TestAlias < Test::Unit::TestCase
@@ -46,12 +47,6 @@ class TestAlias < Test::Unit::TestCase
     assert_raise(NoMethodError) { x.quux }
   end
 
-  class C
-    def m
-      $SAFE
-    end
-  end
-
   def test_nonexistmethod
     assert_raise(NameError){
       Class.new{
@@ -121,7 +116,8 @@ class TestAlias < Test::Unit::TestCase
   end
 
   def test_alias_wb_miss
-    assert_normal_exit %q{
+    assert_normal_exit "#{<<-"begin;"}\n#{<<-'end;'}"
+    begin;
       require 'stringio'
       GC.verify_internal_consistency
       GC.start
@@ -129,7 +125,7 @@ class TestAlias < Test::Unit::TestCase
         alias_method :read_nonblock, :sysread
       end
       GC.verify_internal_consistency
-    }
+    end;
   end
 
   def test_cyclic_zsuper
@@ -182,7 +178,8 @@ class TestAlias < Test::Unit::TestCase
   def test_alias_in_module
     bug9663 = '[ruby-core:61635] [Bug #9663]'
 
-    assert_separately(['-', bug9663], <<-'end;')
+    assert_separately(['-', bug9663], "#{<<-"begin;"}\n#{<<-'end;'}")
+    begin;
       bug = ARGV[0]
 
       m = Module.new do
@@ -193,4 +190,41 @@ class TestAlias < Test::Unit::TestCase
       assert_equal(o.to_s, o.orig_to_s, bug)
     end;
   end
+
+  class C0; def foo; end; end
+  class C1 < C0; alias bar foo; end
+
+  def test_alias_method_equation
+    obj = C1.new
+    assert_equal(obj.method(:bar), obj.method(:foo))
+    assert_equal(obj.method(:foo), obj.method(:bar))
+  end
+
+  def test_alias_class_method_added
+    name = nil
+    k = Class.new {
+      def foo;end
+      def self.method_added(mid)
+        @name = instance_method(mid).original_name
+      end
+      alias bar foo
+      name = @name
+    }
+    assert_equal(:foo, k.instance_method(:bar).original_name)
+    assert_equal(:foo, name)
+  end
+
+  def test_alias_module_method_added
+    name = nil
+    k = Module.new {
+      def foo;end
+      def self.method_added(mid)
+        @name = instance_method(mid).original_name
+      end
+      alias bar foo
+      name = @name
+    }
+    assert_equal(:foo, k.instance_method(:bar).original_name)
+    assert_equal(:foo, name)
+  end
 end
