@@ -1,8 +1,8 @@
+# frozen_string_literal: false
 require 'test/unit'
 require 'fiber'
-require 'continuation'
+EnvUtil.suppress_warning {require 'continuation'}
 require 'tmpdir'
-require_relative './envutil'
 
 class TestFiber < Test::Unit::TestCase
   def test_normal
@@ -34,6 +34,7 @@ class TestFiber < Test::Unit::TestCase
   end
 
   def test_many_fibers
+    skip 'This is unstable on GitHub Actions --jit-wait. TODO: debug it' if RubyVM::MJIT.enabled?
     max = 10_000
     assert_equal(max, max.times{
       Fiber.new{}
@@ -70,10 +71,12 @@ class TestFiber < Test::Unit::TestCase
     assert_raise(ArgumentError){
       Fiber.new # Fiber without block
     }
-    assert_raise(FiberError){
-      f = Fiber.new{}
-      Thread.new{f.resume}.join # Fiber yielding across thread
-    }
+    f = Fiber.new{}
+    Thread.new{
+      assert_raise(FiberError){ # Fiber yielding across thread
+        f.resume
+      }
+    }.join
     assert_raise(FiberError){
       f = Fiber.new{}
       f.resume
@@ -107,6 +110,15 @@ class TestFiber < Test::Unit::TestCase
       }
       fib.resume
     }
+    assert_raise(FiberError){
+      fib = Fiber.new{}
+      fib.raise "raise in unborn fiber"
+    }
+    assert_raise(FiberError){
+      fib = Fiber.new{}
+      fib.resume
+      fib.raise "raise in dead fiber"
+    }
   end
 
   def test_return
@@ -118,13 +130,45 @@ class TestFiber < Test::Unit::TestCase
   end
 
   def test_throw
-    assert_raise(ArgumentError){
+    assert_raise(UncaughtThrowError){
       Fiber.new do
         throw :a
       end.resume
     }
   end
 
+  def test_raise
+    assert_raise(ZeroDivisionError){
+      Fiber.new do
+        1/0
+      end.resume
+    }
+    assert_raise(RuntimeError){
+      fib = Fiber.new{ Fiber.yield }
+      fib.resume
+      fib.raise "raise and propagate"
+    }
+    assert_nothing_raised{
+      fib = Fiber.new do
+        begin
+          Fiber.yield
+        rescue
+        end
+      end
+      fib.resume
+      fib.raise "rescue in fiber"
+    }
+    fib = Fiber.new do
+      begin
+        Fiber.yield
+      rescue
+        Fiber.yield :ok
+      end
+    end
+    fib.resume
+    assert_equal(:ok, fib.raise)
+  end
+
   def test_transfer
     ary = []
     f2 = nil
@@ -199,11 +243,11 @@ class TestFiber < Test::Unit::TestCase
   end
 
   def test_resume_root_fiber
-    assert_raise(FiberError) do
-      Thread.new do
+    Thread.new do
+      assert_raise(FiberError) do
         Fiber.current.resume
-      end.join
-    end
+      end
+    end.join
   end
 
   def test_gc_root_fiber
@@ -217,13 +261,24 @@ class TestFiber < Test::Unit::TestCase
     }, bug4612
   end
 
-  def test_no_valid_cfp
-    bug5083 = '[ruby-dev:44208]'
-    assert_equal([], Fiber.new(&Module.method(:nesting)).resume)
-    assert_instance_of(Class, Fiber.new(&Class.new.method(:undef_method)).resume(:to_s))
+  def test_mark_fiber
+    bug13875 = '[ruby-core:82681]'
+
+    assert_normal_exit %q{
+      GC.stress = true
+      up = 1.upto(10)
+      down = 10.downto(1)
+      up.zip(down) {|a, b| a + b == 11 or fail 'oops'}
+    }, bug13875
   end
 
-  def test_prohibit_resume_transfered_fiber
+  def test_no_valid_cfp
+    bug5083 = '[ruby-dev:44208]'
+    assert_equal([], Fiber.new(&Module.method(:nesting)).resume, bug5083)
+    assert_instance_of(Class, Fiber.new(&Class.new.method(:undef_method)).resume(:to_s), bug5083)
+  end
+
+  def test_prohibit_resume_transferred_fiber
     assert_raise(FiberError){
       root_fiber = Fiber.current
       f = Fiber.new{
@@ -247,16 +302,29 @@ class TestFiber < Test::Unit::TestCase
   end
 
   def test_fork_from_fiber
-    begin
-      pid = Process.fork{}
-    rescue NotImplementedError
-      return
-    else
-      Process.wait(pid)
-    end
+    skip 'fork not supported' unless Process.respond_to?(:fork)
+    pid = nil
     bug5700 = '[ruby-core:41456]'
     assert_nothing_raised(bug5700) do
-      Fiber.new{ pid = fork {} }.resume
+      Fiber.new do
+        pid = fork do
+          xpid = nil
+          Fiber.new {
+            xpid = fork do
+              # enough to trigger GC on old root fiber
+              count = 10000
+              count = 1000 if /openbsd/i =~ RUBY_PLATFORM
+              count.times do
+                Fiber.new {}.transfer
+                Fiber.new { Fiber.yield }
+              end
+              exit!(0)
+            end
+          }.transfer
+          _, status = Process.waitpid2(xpid)
+          exit!(status.success?)
+        end
+      end.resume
     end
     pid, status = Process.waitpid2(pid)
     assert_equal(0, status.exitstatus, bug5700)
@@ -283,8 +351,10 @@ class TestFiber < Test::Unit::TestCase
     env = {}
     env['RUBY_FIBER_VM_STACK_SIZE'] = vm_stack_size.to_s if vm_stack_size
     env['RUBY_FIBER_MACHINE_STACK_SIZE'] = machine_stack_size.to_s if machine_stack_size
-    out, err = Dir.mktmpdir("test_fiber") {|tmpdir|
-      EnvUtil.invoke_ruby([env, '-e', script], '', true, true, chdir: tmpdir)
+    out = Dir.mktmpdir("test_fiber") {|tmpdir|
+      out, err, status = EnvUtil.invoke_ruby([env, '-e', script], '', true, true, chdir: tmpdir, timeout: 30)
+      assert(!status.signaled?, FailDesc[status, nil, err])
+      out
     }
     use_length ? out.length : out
   end
@@ -292,7 +362,7 @@ class TestFiber < Test::Unit::TestCase
   def test_stack_size
     h_default = eval(invoke_rec('p RubyVM::DEFAULT_PARAMS', nil, nil, false))
     h_0 = eval(invoke_rec('p RubyVM::DEFAULT_PARAMS', 0, 0, false))
-    h_large = eval(invoke_rec('p RubyVM::DEFAULT_PARAMS', 1024 * 1024 * 10, 1024 * 1024 * 10, false))
+    h_large = eval(invoke_rec('p RubyVM::DEFAULT_PARAMS', 1024 * 1024 * 5, 1024 * 1024 * 10, false))
 
     assert_operator(h_default[:fiber_vm_stack_size], :>, h_0[:fiber_vm_stack_size])
     assert_operator(h_default[:fiber_vm_stack_size], :<, h_large[:fiber_vm_stack_size])
@@ -305,7 +375,7 @@ class TestFiber < Test::Unit::TestCase
     assert_operator(size_default, :>, 0)
     size_0 = invoke_rec script, 0, nil
     assert_operator(size_default, :>, size_0)
-    size_large = invoke_rec script, 1024 * 1024 * 10, nil
+    size_large = invoke_rec script, 1024 * 1024 * 5, nil
     assert_operator(size_default, :<, size_large)
 
     return if /mswin|mingw/ =~ RUBY_PLATFORM
@@ -344,5 +414,44 @@ class TestFiber < Test::Unit::TestCase
     assert_equal("inner", s2)
     assert_equal(s1, $_, bug7678)
   end
-end
 
+  def test_new_symbol_proc
+    bug = '[ruby-core:80147] [Bug #13313]'
+    assert_ruby_status([], "#{<<-"begin;"}\n#{<<-'end;'}", bug)
+    begin;
+      exit("1" == Fiber.new(&:to_s).resume(1))
+    end;
+  end
+
+  def test_to_s
+    f = Fiber.new do
+      assert_match(/resumed/, f.to_s)
+      Fiber.yield
+    end
+    assert_match(/created/, f.to_s)
+    f.resume
+    assert_match(/suspended/, f.to_s)
+    f.resume
+    assert_match(/terminated/, f.to_s)
+    assert_match(/resumed/, Fiber.current.to_s)
+  end
+
+  def test_create_fiber_in_new_thread
+    ret = Thread.new{
+      Thread.new{
+        Fiber.new{Fiber.yield :ok}.resume
+      }.value
+    }.value
+    assert_equal :ok, ret, '[Bug #14642]'
+  end
+
+  def test_machine_stack_gc
+    assert_normal_exit <<-RUBY, '[Bug #14561]', timeout: 10
+      enum = Enumerator.new { |y| y << 1 }
+      thread = Thread.new { enum.peek }
+      thread.join
+      sleep 5     # pause until thread cache wait time runs out. Native thread exits.
+      GC.start
+    RUBY
+  end
+end
