@@ -1,5 +1,6 @@
 # -*- coding: cp932 -*-
-# This test is in a differnt file than TestUnicodeEscapes
+# frozen_string_literal: false
+# This test is in a different file than TestUnicodeEscapes
 # So that we can have a different coding comment above
 
 require 'test/unit'
@@ -12,14 +13,18 @@ class TestMixedUnicodeEscape < Test::Unit::TestCase
     # 8-bit character escapes are okay.
     assert_equal("B\xFF", "\u0042\xFF")
 
-    # sjis mb chars mixed with Unicode shound not work
+    # sjis mb chars mixed with Unicode should not work
     assert_raise(SyntaxError) { eval %q("\u1234")}
     assert_raise(SyntaxError) { eval %q("\u{1234}")}
 
+    # also should not work for Regexp
+    assert_raise(SyntaxError) { eval %q(/#{"\u1234"}#{""}/)}
+    assert_raise(RegexpError) { eval %q(/\u{1234}#{nil}/)}
+    assert_raise(RegexpError) { eval %q(/#{nil}\u1234/)}
+
     # String interpolation turns into an expression and we get
     # a different kind of error, but we still can't mix these
     assert_raise(Encoding::CompatibilityError) { eval %q("\u{1234}#{nil}")}
     assert_raise(Encoding::CompatibilityError) { eval %q("#{nil}\u1234")}
-
   end
 end
