@@ -1,9 +1,11 @@
-# frozen_string_literal: false
+# frozen_string_literal: true
 require 'rubygems/test_case'
 require 'rubygems/command'
 
 class Gem::Command
+
   public :parser
+
 end
 
 class TestGemCommand < Gem::TestCase
@@ -13,8 +15,9 @@ class TestGemCommand < Gem::TestCase
 
     @xopt = nil
 
+    @common_options = Gem::Command.common_options.dup
     Gem::Command.common_options.clear
-    Gem::Command.common_options <<  [
+    Gem::Command.common_options << [
       ['-x', '--exe', 'Execute'], lambda do |*a|
         @xopt = true
       end
@@ -24,9 +27,14 @@ class TestGemCommand < Gem::TestCase
     @cmd = Gem::Command.new @cmd_name, 'summary'
   end
 
+  def teardown
+    super
+    Gem::Command.common_options.replace @common_options
+  end
+
   def test_self_add_specific_extra_args
     added_args = %w[--all]
-    @cmd.add_option '--all' do |v,o| end
+    @cmd.add_option('--all') { |v,o| }
 
     Gem::Command.add_specific_extra_args @cmd_name, added_args
 
@@ -90,7 +98,7 @@ class TestGemCommand < Gem::TestCase
 
   def test_invoke_with_bad_options
     use_ui @ui do
-      @cmd.when_invoked do true end
+      @cmd.when_invoked { true }
 
       ex = assert_raises OptionParser::InvalidOption do
         @cmd.invoke('-zzz')
@@ -101,7 +109,7 @@ class TestGemCommand < Gem::TestCase
   end
 
   def test_invoke_with_common_options
-    @cmd.when_invoked do true end
+    @cmd.when_invoked { true }
 
     use_ui @ui do
       @cmd.invoke "-x"
@@ -170,12 +178,16 @@ class TestGemCommand < Gem::TestCase
     @cmd.add_option('-f', '--file FILE', 'File option') do |value, options|
       options[:help] = true
     end
+    @cmd.add_option('--silent', 'Silence RubyGems output') do |value, options|
+      options[:silent] = true
+    end
     assert @cmd.handles?(['-x'])
     assert @cmd.handles?(['-h'])
     assert @cmd.handles?(['-h', 'command'])
     assert @cmd.handles?(['--help', 'command'])
     assert @cmd.handles?(['-f', 'filename'])
     assert @cmd.handles?(['--file=filename'])
+    assert @cmd.handles?(['--silent'])
     refute @cmd.handles?(['-z'])
     refute @cmd.handles?(['-f'])
     refute @cmd.handles?(['--toothpaste'])
@@ -185,6 +197,122 @@ class TestGemCommand < Gem::TestCase
     assert_equal ['-h', 'command'], args
   end
 
+  def test_deprecate_option
+    deprecate_msg = <<-EXPECTED
+WARNING:  The \"--test\" option has been deprecated and will be removed in Rubygems 3.1.
+    EXPECTED
+
+    testCommand = Class.new(Gem::Command) do
+      def initialize
+        super('test', 'Gem::Command instance for testing')
+
+        add_option('-t', '--test', 'Test command') do |value, options|
+          options[:test] = true
+        end
+
+        deprecate_option('--test', version: '3.1')
+      end
+
+      def execute
+        true
+      end
+    end
+
+    cmd = testCommand.new
+
+    use_ui @ui do
+      cmd.invoke("--test")
+      assert_equal deprecate_msg, @ui.error
+    end
+  end
+
+  def test_deprecate_option_no_version
+    deprecate_msg = <<-EXPECTED
+WARNING:  The \"--test\" option has been deprecated and will be removed in future versions of Rubygems.
+    EXPECTED
+
+    testCommand = Class.new(Gem::Command) do
+      def initialize
+        super('test', 'Gem::Command instance for testing')
+
+        add_option('-t', '--test', 'Test command') do |value, options|
+          options[:test] = true
+        end
+
+        deprecate_option('--test')
+      end
+
+      def execute
+        true
+      end
+    end
+
+    cmd = testCommand.new
+
+    use_ui @ui do
+      cmd.invoke("--test")
+      assert_equal deprecate_msg, @ui.error
+    end
+  end
+
+  def test_deprecate_option_extra_message
+    deprecate_msg = <<-EXPECTED
+WARNING:  The \"--test\" option has been deprecated and will be removed in Rubygems 3.1. Whether you set `--test` mode or not, this dummy app always runs in test mode.
+    EXPECTED
+
+    testCommand = Class.new(Gem::Command) do
+      def initialize
+        super('test', 'Gem::Command instance for testing')
+
+        add_option('-t', '--test', 'Test command') do |value, options|
+          options[:test] = true
+        end
+
+        deprecate_option('--test', version: '3.1', extra_msg: 'Whether you set `--test` mode or not, this dummy app always runs in test mode.')
+      end
+
+      def execute
+        true
+      end
+    end
+
+    cmd = testCommand.new
+
+    use_ui @ui do
+      cmd.invoke("--test")
+      assert_equal deprecate_msg, @ui.error
+    end
+  end
+
+  def test_deprecate_option_extra_message_and_no_version
+    deprecate_msg = <<-EXPECTED
+WARNING:  The \"--test\" option has been deprecated and will be removed in future versions of Rubygems. Whether you set `--test` mode or not, this dummy app always runs in test mode.
+    EXPECTED
+
+    testCommand = Class.new(Gem::Command) do
+      def initialize
+        super('test', 'Gem::Command instance for testing')
+
+        add_option('-t', '--test', 'Test command') do |value, options|
+          options[:test] = true
+        end
+
+        deprecate_option('--test', extra_msg: 'Whether you set `--test` mode or not, this dummy app always runs in test mode.')
+      end
+
+      def execute
+        true
+      end
+    end
+
+    cmd = testCommand.new
+
+    use_ui @ui do
+      cmd.invoke("--test")
+      assert_equal deprecate_msg, @ui.error
+    end
+  end
+
   def test_show_lookup_failure_suggestions_local
     correct    = "non_existent_with_hint"
     misspelled = "nonexistent_with_hint"
@@ -229,7 +357,7 @@ ERROR:  Could not find a valid gem 'other' (>= 0) in any repository
     end
 
     use_ui @ui do
-      @cmd.show_lookup_failure misspelled, Gem::Requirement.default, [], :remote
+      @cmd.show_lookup_failure misspelled, Gem::Requirement.default, []
     end
 
     expected = <<-EXPECTED
@@ -241,4 +369,3 @@ ERROR:  Possible alternatives: non_existent_with_hint
   end
 
 end
-
