@@ -1,4 +1,4 @@
-# frozen_string_literal: false
+# frozen_string_literal: true
 require 'rubygems/test_case'
 require 'rubygems/commands/environment_command'
 
@@ -23,12 +23,13 @@ class TestGemCommandsEnvironmentCommand < Gem::TestCase
     end
 
     assert_match %r|RUBYGEMS VERSION: (\d\.)+\d|, @ui.output
-    assert_match %r|RUBY VERSION: \d\.\d\.\d \(.*\) \[.*\]|, @ui.output
+    assert_match %r|RUBY VERSION: \d+\.\d+\.\d+ \(.*\) \[.*\]|, @ui.output
     assert_match %r|INSTALLATION DIRECTORY: #{Regexp.escape @gemhome}|,
                  @ui.output
     assert_match %r|RUBYGEMS PREFIX: |, @ui.output
     assert_match %r|RUBY EXECUTABLE:.*#{RbConfig::CONFIG['ruby_install_name']}|,
                  @ui.output
+    assert_match %r|GIT EXECUTABLE: #{@cmd.send(:git_path)}|, @ui.output
     assert_match %r|SYSTEM CONFIGURATION DIRECTORY:|, @ui.output
     assert_match %r|EXECUTABLE DIRECTORY:|, @ui.output
     assert_match %r|RUBYGEMS PLATFORMS:|, @ui.output
@@ -89,17 +90,6 @@ class TestGemCommandsEnvironmentCommand < Gem::TestCase
     assert_equal '', @ui.error
   end
 
-  def test_execute_packageversion
-    @cmd.send :handle_options, %w[packageversion]
-
-    use_ui @ui do
-      @cmd.execute
-    end
-
-    assert_equal "#{Gem::RubyGemsPackageVersion}\n", @ui.output
-    assert_equal '', @ui.error
-  end
-
   def test_execute_remotesources
     orig_sources = Gem.sources.dup
     Gem.sources.replace %w[http://gems.example.com]
@@ -151,4 +141,5 @@ class TestGemCommandsEnvironmentCommand < Gem::TestCase
     assert_equal "#{Gem.platforms.join File::PATH_SEPARATOR}\n", @ui.output
     assert_equal '', @ui.error
   end
+
 end
