@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'rubygems/test_case'
 require 'rubygems/dependency_installer'
 require 'rubygems/security'
@@ -14,25 +15,36 @@ class TestGemDependencyInstaller < Gem::TestCase
     FileUtils.mkdir @gems_dir
 
     Gem::RemoteFetcher.fetcher = @fetcher = Gem::FakeFetcher.new
+
+    @original_platforms = Gem.platforms
+    Gem.platforms = []
+  end
+
+  def teardown
+    Gem.platforms = @original_platforms
+    super
   end
 
   def util_setup_gems
-    @a1, @a1_gem         = util_gem 'a', '1' do |s| s.executables << 'a_bin' end
+    @a1, @a1_gem = util_gem 'a', '1' do |s|
+      s.executables << 'a_bin'
+    end
+
     @a1_pre, @a1_pre_gem = util_gem 'a', '1.a'
-    @b1, @b1_gem         = util_gem 'b', '1' do |s|
+
+    @b1, @b1_gem = util_gem 'b', '1' do |s|
       s.add_dependency 'a'
       s.add_development_dependency 'aa'
     end
 
-    @c1, @c1_gem         = util_gem 'c', '1' do |s|
+    @c1, @c1_gem = util_gem 'c', '1' do |s|
       s.add_development_dependency 'b'
     end
 
-    @d1, @d1_gem         = util_gem 'd', '1' do |s|
+    @d1, @d1_gem = util_gem 'd', '1' do |s|
       s.add_development_dependency 'c'
     end
 
-    util_clear_gems
     util_reset_gems
   end
 
@@ -43,7 +55,9 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     inst = Gem::DependencyInstaller.new
 
-    available = inst.available_set_for 'a', Gem::Requirement.default
+    available = Gem::Deprecate.skip_during do
+      inst.available_set_for 'a', Gem::Requirement.default
+    end
 
     assert_equal %w[a-1], available.set.map { |s| s.spec.full_name }
   end
@@ -55,7 +69,9 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     inst = Gem::DependencyInstaller.new :prerelease => true
 
-    available = inst.available_set_for 'a', Gem::Requirement.default
+    available = Gem::Deprecate.skip_during do
+      inst.available_set_for 'a', Gem::Requirement.default
+    end
 
     assert_equal %w[a-10.a],
                  available.sorted.map { |s| s.spec.full_name }
@@ -70,7 +86,9 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     dep = Gem::Dependency.new 'a', Gem::Requirement.default
 
-    available = inst.available_set_for dep, Gem::Requirement.default
+    available = Gem::Deprecate.skip_during do
+      inst.available_set_for dep, Gem::Requirement.default
+    end
 
     assert_equal %w[a-1], available.set.map { |s| s.spec.full_name }
   end
@@ -85,7 +103,9 @@ class TestGemDependencyInstaller < Gem::TestCase
     dep = Gem::Dependency.new 'a', Gem::Requirement.default
     dep.prerelease = true
 
-    available = inst.available_set_for dep, Gem::Requirement.default
+    available = Gem::Deprecate.skip_during do
+      inst.available_set_for dep, Gem::Requirement.default
+    end
 
     assert_equal %w[a-10.a],
                  available.sorted.map { |s| s.spec.full_name }
@@ -112,7 +132,6 @@ class TestGemDependencyInstaller < Gem::TestCase
     p1a, gem = util_gem 'a', '10.a'
 
     util_setup_spec_fetcher(p1a, @a1, @a1_pre)
-    util_clear_gems
 
     p1a_data = Gem.read_binary(gem)
 
@@ -151,7 +170,6 @@ class TestGemDependencyInstaller < Gem::TestCase
     p1a, gem = util_gem 'p', '1.a'
 
     util_setup_spec_fetcher(p1a)
-    util_clear_gems
 
     p1a_data = Gem.read_binary(gem)
 
@@ -171,7 +189,6 @@ class TestGemDependencyInstaller < Gem::TestCase
     util_setup_gems
 
     util_setup_spec_fetcher(@a1, @a1_pre)
-    util_clear_gems
 
     p1a_data = Gem.read_binary(@a1_gem)
 
@@ -192,8 +209,6 @@ class TestGemDependencyInstaller < Gem::TestCase
       s.add_dependency 'b'
     end
 
-    util_clear_gems
-
     FileUtils.mv @a1_gem, @tempdir
     FileUtils.mv @b1_gem, @tempdir
     FileUtils.mv  e1_gem, @tempdir
@@ -272,7 +287,7 @@ class TestGemDependencyInstaller < Gem::TestCase
   # This asserts that if a gem's dependency is satisfied by an
   # already installed gem, RubyGems doesn't installed a newer
   # version
-  def test_install_doesnt_upgrade_installed_depedencies
+  def test_install_doesnt_upgrade_installed_dependencies
     util_setup_gems
 
     a2, a2_gem = util_gem 'a', '2'
@@ -398,7 +413,7 @@ class TestGemDependencyInstaller < Gem::TestCase
   def test_install_dependency_existing
     util_setup_gems
 
-    Gem::Installer.new(@a1_gem).install
+    Gem::Installer.at(@a1_gem).install
     FileUtils.mv @a1_gem, @tempdir
     FileUtils.mv @b1_gem, @tempdir
     inst = nil
@@ -415,30 +430,34 @@ class TestGemDependencyInstaller < Gem::TestCase
     extconf_rb = File.join @gemhome, 'gems', 'e-1', 'extconf.rb'
     FileUtils.mkdir_p File.dirname extconf_rb
 
-    open extconf_rb, 'w' do |io|
+    File.open extconf_rb, 'w' do |io|
       io.write <<-EXTCONF_RB
         require 'mkmf'
         create_makefile 'e'
       EXTCONF_RB
     end
 
-    e1 = new_spec 'e', '1', nil, 'extconf.rb' do |s|
+    e1 = util_spec 'e', '1', nil, 'extconf.rb' do |s|
       s.extensions << 'extconf.rb'
     end
     e1_gem = File.join @tempdir, 'gems', "#{e1.full_name}.gem"
 
     _, f1_gem = util_gem 'f', '1', 'e' => nil
 
-    Gem::Installer.new(e1_gem).install
+    Gem::Installer.at(e1_gem).install
     FileUtils.rm_r e1.extension_dir
 
     FileUtils.mv e1_gem, @tempdir
     FileUtils.mv f1_gem, @tempdir
     inst = nil
 
-    Dir.chdir @tempdir do
+    pwd = Dir.getwd
+    Dir.chdir @tempdir
+    begin
       inst = Gem::DependencyInstaller.new
       inst.install 'f'
+    ensure
+      Dir.chdir pwd
     end
 
     assert_equal %w[f-1], inst.installed_gems.map { |s| s.full_name }
@@ -517,7 +536,7 @@ class TestGemDependencyInstaller < Gem::TestCase
     inst = nil
 
     Dir.chdir @tempdir do
-      Gem::Installer.new('a-1.gem').install
+      Gem::Installer.at('a-1.gem').install
 
       inst = Gem::DependencyInstaller.new :domain => :local
       inst.install 'b-1.gem'
@@ -550,8 +569,6 @@ class TestGemDependencyInstaller < Gem::TestCase
       s.add_dependency 'a'
     end
 
-    util_clear_gems
-
     FileUtils.mv @a1_gem, @tempdir
     FileUtils.mv @b1_gem, @tempdir
     FileUtils.mv  b2_gem, @tempdir
@@ -661,7 +678,7 @@ class TestGemDependencyInstaller < Gem::TestCase
     FileUtils.mv @a1_gem, @tempdir
     FileUtils.mv @b1_gem, @tempdir
 
-    inst = Gem::Installer.new @a1.file_name
+    inst = Gem::Installer.at @a1.file_name
     inst.install
 
     gemhome2 = File.join @tempdir, 'gemhome2'
@@ -791,7 +808,7 @@ class TestGemDependencyInstaller < Gem::TestCase
   def test_install_reinstall
     util_setup_gems
 
-    Gem::Installer.new(@a1_gem).install
+    Gem::Installer.at(@a1_gem).install
     FileUtils.mv @a1_gem, @tempdir
     inst = nil
 
@@ -852,15 +869,18 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     si = util_setup_spec_fetcher @a1, a2_o
 
-    util_clear_gems
-
     @fetcher.data['http://gems.example.com/gems/yaml'] = si.to_yaml
 
     a1_data = nil
     a2_o_data = nil
 
-    File.open @a1_gem, 'rb' do |fp| a1_data = fp.read end
-    File.open a2_o_gem, 'rb' do |fp| a2_o_data = fp.read end
+    File.open @a1_gem, 'rb' do |fp|
+      a1_data = fp.read
+    end
+
+    File.open a2_o_gem, 'rb' do |fp|
+      a2_o_data = fp.read
+    end
 
     @fetcher.data["http://gems.example.com/gems/#{@a1.file_name}"] =
       a1_data
@@ -884,7 +904,7 @@ class TestGemDependencyInstaller < Gem::TestCase
     assert_equal %w[a-1-cpu-other_platform-1], inst.installed_gems.map { |s| s.full_name }
   end
 
-  if defined? OpenSSL then
+  if defined? OpenSSL
     def test_install_security_policy
       util_setup_gems
 
@@ -909,7 +929,7 @@ class TestGemDependencyInstaller < Gem::TestCase
   end
 
   # Wrappers don't work on mswin
-  unless win_platform? then
+  unless win_platform?
     def test_install_no_wrappers
       util_setup_gems
 
@@ -962,7 +982,9 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     Gem::Specification.reset
 
-    set = inst.find_gems_with_sources(dep)
+    set = Gem::Deprecate.skip_during do
+      inst.find_gems_with_sources(dep)
+    end
 
     assert_kind_of Gem::AvailableSet, set
 
@@ -980,7 +1002,9 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     inst = Gem::DependencyInstaller.new
 
-    available = inst.find_spec_by_name_and_version('*.gem')
+    available = Gem::Deprecate.skip_during do
+      inst.find_spec_by_name_and_version('*.gem')
+    end
 
     assert_equal %w[a-1], available.each_spec.map { |spec| spec.full_name }
   end
@@ -991,7 +1015,9 @@ class TestGemDependencyInstaller < Gem::TestCase
     inst = Gem::DependencyInstaller.new
 
     assert_raises Gem::Package::FormatError do
-      inst.find_spec_by_name_and_version '*.gem'
+      Gem::Deprecate.skip_during do
+        inst.find_spec_by_name_and_version '*.gem'
+      end
     end
   end
 
@@ -1001,7 +1027,9 @@ class TestGemDependencyInstaller < Gem::TestCase
     inst = Gem::DependencyInstaller.new
 
     e = assert_raises Gem::Package::FormatError do
-      inst.find_spec_by_name_and_version 'rdoc.gem'
+      Gem::Deprecate.skip_during do
+        inst.find_spec_by_name_and_version 'rdoc.gem'
+      end
     end
 
     full_path = File.join @tempdir, 'rdoc.gem'
@@ -1014,7 +1042,9 @@ class TestGemDependencyInstaller < Gem::TestCase
     inst = Gem::DependencyInstaller.new
 
     e = assert_raises Gem::SpecificGemNotFoundException do
-      inst.find_spec_by_name_and_version 'rdoc'
+      Gem::Deprecate.skip_during do
+        inst.find_spec_by_name_and_version 'rdoc'
+      end
     end
 
     assert_equal "Could not find a valid gem 'rdoc' (>= 0) " +
@@ -1028,7 +1058,9 @@ class TestGemDependencyInstaller < Gem::TestCase
     inst = Gem::DependencyInstaller.new
 
     e = assert_raises Gem::SpecificGemNotFoundException do
-      inst.find_spec_by_name_and_version 'rdoc'
+      Gem::Deprecate.skip_during do
+        inst.find_spec_by_name_and_version 'rdoc'
+      end
     end
 
     assert_equal "Could not find a valid gem 'rdoc' (>= 0) " +
@@ -1045,7 +1077,9 @@ class TestGemDependencyInstaller < Gem::TestCase
     set = nil
 
     Dir.chdir @tempdir do
-      set = inst.find_gems_with_sources dep
+      set = Gem::Deprecate.skip_during do
+        inst.find_gems_with_sources dep
+      end
     end
 
     gems = set.sorted
@@ -1060,7 +1094,6 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     assert_equal 'a-1', remote.spec.full_name, 'remote spec'
     assert_equal Gem::Source.new(@gem_repo), remote.source, 'remote path'
-
   end
 
   def test_find_gems_with_sources_prerelease
@@ -1070,20 +1103,47 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     dependency = Gem::Dependency.new('a', Gem::Requirement.default)
 
-    releases =
-      installer.find_gems_with_sources(dependency).all_specs
+    set = Gem::Deprecate.skip_during do
+      installer.find_gems_with_sources(dependency)
+    end
+
+    releases = set.all_specs
 
     assert releases.any? { |s| s.name == 'a' and s.version.to_s == '1' }
     refute releases.any? { |s| s.name == 'a' and s.version.to_s == '1.a' }
 
     dependency.prerelease = true
 
-    prereleases =
-      installer.find_gems_with_sources(dependency).all_specs
+    set = Gem::Deprecate.skip_during do
+      installer.find_gems_with_sources(dependency)
+    end
+
+    prereleases = set.all_specs
 
     assert_equal [@a1_pre, @a1], prereleases
   end
 
+  def test_find_gems_with_sources_with_best_only_and_platform
+    util_setup_gems
+    a1_x86_mingw32, = util_gem 'a', '1' do |s|
+      s.platform = 'x86-mingw32'
+    end
+    util_setup_spec_fetcher @a1, a1_x86_mingw32
+    Gem.platforms << Gem::Platform.new('x86-mingw32')
+
+    installer = Gem::DependencyInstaller.new
+
+    dependency = Gem::Dependency.new('a', Gem::Requirement.default)
+
+    set = Gem::Deprecate.skip_during do
+      installer.find_gems_with_sources(dependency, true)
+    end
+
+    releases = set.all_specs
+
+    assert_equal [a1_x86_mingw32], releases
+  end
+
   def test_find_gems_with_sources_with_bad_source
     Gem.sources.replace ["http://not-there.nothing"]
 
@@ -1091,7 +1151,9 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     dep = Gem::Dependency.new('a')
 
-    out = installer.find_gems_with_sources(dep)
+    out = Gem::Deprecate.skip_during do
+      installer.find_gems_with_sources(dep)
+    end
 
     assert out.empty?
     assert_kind_of Gem::SourceFetchProblem, installer.errors.first
@@ -1202,7 +1264,6 @@ class TestGemDependencyInstaller < Gem::TestCase
     util_setup_spec_fetcher(*[@a1, @a1_pre, @b1, @c1_pre,
                               @d1, @d2, @x1_m, @x1_o, @w1, @y1,
                               @y1_1_p, @z1].compact)
-
-    util_clear_gems
   end
+
 end
