@@ -8,6 +8,8 @@ class TestGemGemcutterUtilities < Gem::TestCase
 
   def setup
     super
+    # below needed for random testing, class property
+    Gem.configuration.disable_default_gem_server = nil
 
     ENV['RUBYGEMS_HOST'] = nil
     Gem.configuration.rubygems_api_key = nil
@@ -31,7 +33,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
 
     FileUtils.mkdir_p File.dirname Gem.configuration.credentials_path
 
-    open Gem.configuration.credentials_path, 'w' do |f|
+    File.open Gem.configuration.credentials_path, 'w' do |f|
       f.write keys.to_yaml
     end
 
@@ -46,7 +48,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
     keys = { :rubygems_api_key => 'KEY' }
     FileUtils.mkdir_p File.dirname Gem.configuration.credentials_path
 
-    open Gem.configuration.credentials_path, 'w' do |f|
+    File.open Gem.configuration.credentials_path, 'w' do |f|
       f.write keys.to_yaml
     end
 
@@ -59,7 +61,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
     keys = { :rubygems_api_key => 'KEY', :other => 'OTHER' }
     FileUtils.mkdir_p File.dirname Gem.configuration.credentials_path
 
-    open Gem.configuration.credentials_path, 'w' do |f|
+    File.open Gem.configuration.credentials_path, 'w' do |f|
       f.write keys.to_yaml
     end
 
@@ -88,7 +90,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
   end
 
   def test_sign_in
-    api_key     = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
+    api_key = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
     util_sign_in [api_key, 200, 'OK']
 
     assert_match %r{Enter your RubyGems.org credentials.}, @sign_in_ui.output
@@ -100,7 +102,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
   end
 
   def test_sign_in_with_host
-    api_key     = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
+    api_key = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
 
     util_sign_in [api_key, 200, 'OK'], 'http://example.com', ['http://example.com']
 
@@ -114,7 +116,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
   end
 
   def test_sign_in_with_host_nil
-    api_key     = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
+    api_key = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
 
     util_sign_in [api_key, 200, 'OK'], nil, [nil]
 
@@ -128,7 +130,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
   end
 
   def test_sign_in_with_host_ENV
-    api_key     = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
+    api_key = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
     util_sign_in [api_key, 200, 'OK'], 'http://example.com'
 
     assert_match "Enter your http://example.com credentials.",
@@ -141,7 +143,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
   end
 
   def test_sign_in_skips_with_existing_credentials
-    api_key     = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
+    api_key = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
     Gem.configuration.rubygems_api_key = api_key
 
     util_sign_in [api_key, 200, 'OK']
@@ -150,8 +152,8 @@ class TestGemGemcutterUtilities < Gem::TestCase
   end
 
   def test_sign_in_skips_with_key_override
-    api_key     = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
-    Gem.configuration.api_keys[:KEY]  = 'other'
+    api_key = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
+    Gem.configuration.api_keys[:KEY] = 'other'
     @cmd.options[:key] = :KEY
     util_sign_in [api_key, 200, 'OK']
 
@@ -163,7 +165,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
     other_api_key = 'f46dbb18bb6a9c97cdc61b5b85c186a17403cdcbf'
 
     FileUtils.mkdir_p File.dirname(Gem.configuration.credentials_path)
-    open Gem.configuration.credentials_path, 'w' do |f|
+    File.open Gem.configuration.credentials_path, 'w' do |f|
       f.write Hash[:other_api_key, other_api_key].to_yaml
     end
     util_sign_in [api_key, 200, 'OK']
@@ -171,14 +173,12 @@ class TestGemGemcutterUtilities < Gem::TestCase
     assert_match %r{Enter your RubyGems.org credentials.}, @sign_in_ui.output
     assert_match %r{Signed in.}, @sign_in_ui.output
 
-    credentials   = YAML.load_file Gem.configuration.credentials_path
+    credentials = YAML.load_file Gem.configuration.credentials_path
     assert_equal api_key, credentials[:rubygems_api_key]
     assert_equal other_api_key, credentials[:other_api_key]
   end
 
   def test_sign_in_with_bad_credentials
-    skip 'Always uses $stdin on windows' if Gem.win_platform?
-
     assert_raises Gem::MockGemUi::TermError do
       util_sign_in ['Access Denied.', 403, 'Forbidden']
     end
@@ -187,9 +187,35 @@ class TestGemGemcutterUtilities < Gem::TestCase
     assert_match %r{Access Denied.}, @sign_in_ui.output
   end
 
-  def util_sign_in response, host = nil, args = []
-    skip 'Always uses $stdin on windows' if Gem.win_platform?
+  def test_sign_in_with_correct_otp_code
+    api_key       = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
+    response_fail = "You have enabled multifactor authentication but your request doesn't have the correct OTP code. Please check it and retry."
 
+    util_sign_in(proc do
+      @call_count ||= 0
+      (@call_count += 1).odd? ? [response_fail, 401, 'Unauthorized'] : [api_key, 200, 'OK']
+    end, nil, [], "111111\n")
+
+    assert_match 'You have enabled multi-factor authentication. Please enter OTP code.', @sign_in_ui.output
+    assert_match 'Code: ', @sign_in_ui.output
+    assert_match 'Signed in.', @sign_in_ui.output
+    assert_equal '111111', @fetcher.last_request['OTP']
+  end
+
+  def test_sign_in_with_incorrect_otp_code
+    response = "You have enabled multifactor authentication but your request doesn't have the correct OTP code. Please check it and retry."
+
+    assert_raises Gem::MockGemUi::TermError do
+      util_sign_in [response, 401, 'Unauthorized'], nil, [], "111111\n"
+    end
+
+    assert_match 'You have enabled multi-factor authentication. Please enter OTP code.', @sign_in_ui.output
+    assert_match 'Code: ', @sign_in_ui.output
+    assert_match response, @sign_in_ui.output
+    assert_equal '111111', @fetcher.last_request['OTP']
+  end
+
+  def util_sign_in(response, host = nil, args = [], extra_input = '')
     email    = 'you@example.com'
     password = 'secret'
 
@@ -203,10 +229,10 @@ class TestGemGemcutterUtilities < Gem::TestCase
     @fetcher.data["#{host}/api/v1/api_key"] = response
     Gem::RemoteFetcher.fetcher = @fetcher
 
-    @sign_in_ui = Gem::MockGemUi.new "#{email}\n#{password}\n"
+    @sign_in_ui = Gem::MockGemUi.new("#{email}\n#{password}\n" + extra_input)
 
     use_ui @sign_in_ui do
-      if args.length > 0 then
+      if args.length > 0
         @cmd.sign_in(*args)
       else
         @cmd.sign_in
