@@ -5,6 +5,7 @@ require 'rubygems/util'
 class TestGemUtil < Gem::TestCase
 
   def test_class_popen
+    skip "popen with a block does not behave well on jruby" if Gem.java_platform?
     assert_equal "0\n", Gem::Util.popen(Gem.ruby, '-I', File.expand_path('../../../lib', __FILE__), '-e', 'p 0')
 
     assert_raises Errno::ECHILD do
@@ -13,6 +14,7 @@ class TestGemUtil < Gem::TestCase
   end
 
   def test_silent_system
+    skip if Gem.java_platform?
     assert_silent do
       Gem::Util.silent_system Gem.ruby, '-I', File.expand_path('../../../lib', __FILE__), '-e', 'puts "hello"; warn "hello"'
     end
@@ -30,7 +32,7 @@ class TestGemUtil < Gem::TestCase
   end
 
   def test_traverse_parents_does_not_crash_on_permissions_error
-    skip 'skipped on MS Windows (chmod has no effect)' if win_platform?
+    skip 'skipped on MS Windows (chmod has no effect)' if win_platform? || java_platform?
 
     FileUtils.mkdir_p 'd/e/f'
     # remove 'execute' permission from "e" directory and make it
@@ -47,13 +49,13 @@ class TestGemUtil < Gem::TestCase
     assert_equal File.realpath("..", Dir.tmpdir), paths[3]
   ensure
     # restore default permissions, allow the directory to be removed
-    FileUtils.chmod(0775, 'd/e') unless win_platform?
+    FileUtils.chmod(0775, 'd/e') unless win_platform? || java_platform?
   end
 
   def test_linked_list_find
-    list = [1,2,3,4,5].inject(Gem::List.new(0)) { |m,o|
+    list = [1,2,3,4,5].inject(Gem::List.new(0)) do |m,o|
       Gem::List.new o, m
-    }
+    end
     assert_equal 5, list.find { |x| x == 5 }
     assert_equal 4, list.find { |x| x == 4 }
   end
@@ -75,4 +77,12 @@ class TestGemUtil < Gem::TestCase
     assert_equal expected_paths.to_set, files_with_relative_base.to_set
   end
 
+  def test_correct_for_windows_path
+    path = "/C:/WINDOWS/Temp/gems"
+    assert_equal "C:/WINDOWS/Temp/gems", Gem::Util.correct_for_windows_path(path)
+
+    path = "/home/skillet"
+    assert_equal "/home/skillet", Gem::Util.correct_for_windows_path(path)
+  end
+
 end
