@@ -52,6 +52,17 @@ class TestTmpdir < Test::Unit::TestCase
     }
   end
 
+  def test_mktmpdir_mutate
+    bug16918 = '[ruby-core:98563]'
+    assert_nothing_raised(bug16918) do
+      assert_mktmpdir_traversal do |traversal_path|
+        Dir.mktmpdir(traversal_path + 'foo') do |actual|
+          actual << "foo"
+        end
+      end
+    end
+  end
+
   def test_mktmpdir_traversal
     assert_mktmpdir_traversal do |traversal_path|
       Dir.mktmpdir(traversal_path + 'foo') do |actual|
@@ -75,6 +86,10 @@ class TestTmpdir < Test::Unit::TestCase
       traversal_path = Array.new(target.count('/')-2, '..').join('/') + traversal_path
       actual = yield traversal_path
       assert_not_send([File.absolute_path(actual), :start_with?, target])
+      [File::SEPARATOR, File::ALT_SEPARATOR].compact.each do |separator|
+        actual = yield traversal_path.tr('/', separator)
+        assert_not_send([File.absolute_path(actual), :start_with?, target])
+      end
     end
   end
 end
