@@ -1,11 +1,14 @@
 #!/usr/bin/ruby -pi
 BEGIN {
+  # pathological setting
+  ENV['LANG'] = ENV['LC_MESSAGES'] = ENV['LC_ALL'] = 'C'
+
   require_relative 'lib/colorize'
 
   colorize = Colorize.new
   file = ARGV.shift
   begin
-    version = IO.popen(ARGV+%w[--version], &:read)
+    version = IO.popen(ARGV+%w[--version], "rb", &:read)
   rescue Errno::ENOENT
     abort "Failed to run `#{colorize.fail ARGV.join(' ')}'; You may have to install it."
   end
@@ -18,3 +21,4 @@ BEGIN {
   ARGV.push(file)
 }
 $_.sub!(/^%define\s+api\.pure/, '%pure-parser')
+$_.sub!(/^%define\s+.*/, '')
