@@ -63,7 +63,7 @@ class ArrayCode
   end
 
   def insert_at_last(num, str)
-    newnum = self.length + num
+    # newnum = self.length + num
     @content << str
     @len += num
   end
@@ -145,7 +145,7 @@ class ActionMap
                 else
                   b = $1.to_i(16)
                   e = $2.to_i(16)
-                  b.upto(e) {|c| set[c] = true }
+                  b.upto(e) {|_| set[_] = true }
                 end
               }
               i = nil
@@ -235,7 +235,7 @@ class ActionMap
     all_rects = []
 
     rects1.each {|rect|
-      min, max, action = rect
+      _, _, action = rect
       rect[2] = actions.length
       actions << action
       all_rects << rect
@@ -244,7 +244,7 @@ class ActionMap
     boundary = actions.length
 
     rects2.each {|rect|
-      min, max, action = rect
+      _, _, action = rect
       rect[2] = actions.length
       actions << action
       all_rects << rect
@@ -273,7 +273,7 @@ class ActionMap
     singleton_rects = []
     region_rects = []
     rects.each {|rect|
-      min, max, action = rect
+      min, max, = rect
       if min == max
         singleton_rects << rect
       else
@@ -293,14 +293,14 @@ class ActionMap
     if region_rects.empty? ? s_rect[0].length == prefix.length : region_rects[0][0].empty?
       h = TMPHASH
       while (s_rect = @singleton_rects.last) && s_rect[0].start_with?(prefix)
-        min, max, action = @singleton_rects.pop
+        min, _, action = @singleton_rects.pop
         raise ArgumentError, "ambiguous pattern: #{prefix}" if min.length != prefix.length
         h[action] = true
       end
-      region_rects.each {|min, max, action|
+      for min, _, action in region_rects
         raise ArgumentError, "ambiguous pattern: #{prefix}" if !min.empty?
         h[action] = true
-      }
+      end
       tree = Action.new(block.call(prefix, h.keys))
       h.clear
     else
@@ -725,13 +725,13 @@ def citrus_decode_mapsrc(ces, csid, mapsrcs)
     path << ".src"
     path[path.rindex('/')] = '%'
     STDERR.puts 'load mapsrc %s' % path if VERBOSE_MODE
-    open(path) do |f|
+    open(path, 'rb') do |f|
       f.each_line do |l|
         break if /^BEGIN_MAP/ =~ l
       end
       f.each_line do |l|
         next if /^\s*(?:#|$)/ =~ l
-          break if /^END_MAP/ =~ l
+        break if /^END_MAP/ =~ l
         case mode
         when :from_ucs
           case l
@@ -740,14 +740,14 @@ def citrus_decode_mapsrc(ces, csid, mapsrcs)
           when /(0x\w+)\s*=\s*(0x\w+)/
             table.push << [plane | $1.hex, citrus_cstomb(ces, csid, $2.hex)]
           else
-            raise "unknown notation '%s'"% l
+            raise "unknown notation '%s'"% l.chomp
           end
         when :to_ucs
           case l
           when /(0x\w+)\s*=\s*(0x\w+)/
             table.push << [citrus_cstomb(ces, csid, $1.hex), plane | $2.hex]
           else
-            raise "unknown notation '%s'"% l
+            raise "unknown notation '%s'"% l.chomp
           end
         end
       end
@@ -906,7 +906,7 @@ end
 
 def transcode_generate_node(am, name_hint=nil)
   STDERR.puts "converter for #{name_hint}" if VERBOSE_MODE
-  name = am.gennode(TRANSCODE_GENERATED_BYTES_CODE, TRANSCODE_GENERATED_WORDS_CODE, name_hint)
+  am.gennode(TRANSCODE_GENERATED_BYTES_CODE, TRANSCODE_GENERATED_WORDS_CODE, name_hint)
   ''
 end
 
@@ -1031,7 +1031,7 @@ if __FILE__ == $0
   VERBOSE_MODE = verbose_mode
 
   OUTPUT_FILENAME = output_filename
-  OUTPUT_PREFIX = output_filename ? File.basename(output_filename)[/\A[A-Za-z0-9_]*/] : ""
+  OUTPUT_PREFIX = output_filename ? File.basename(output_filename)[/\A[A-Za-z0-9_]*/] : "".dup
   OUTPUT_PREFIX.sub!(/\A_+/, '')
   OUTPUT_PREFIX.sub!(/_*\z/, '_')
 
@@ -1078,7 +1078,11 @@ if __FILE__ == $0
   end
 
   libs1 = $".dup
-  erb = ERB.new(src, nil, '%')
+  if ERB.instance_method(:initialize).parameters.assoc(:key) # Ruby 2.6+
+    erb = ERB.new(src, trim_mode: '%')
+  else
+    erb = ERB.new(src, nil, '%')
+  end
   erb.filename = arg
   erb_result = erb.result(binding)
   libs2 = $".dup
